/****************************************************************************/
/*!
   	@file		Array1_pub.h
 	
	@author 	Fanzo
 	@date 		2007-9-12
*/
/****************************************************************************/

#ifndef		Array1_pub_hpp
#define		Array1_pub_hpp

///////////////////////////////////////////////////////////////////////////
//include files
#include	"Real.h"

namespace Array1
{
using namespace real;
///////////////////////////////////////////////////////////////////////////
// preprocessor deifne

///////////////////////////////////////////////////////////////////////////
// type define
enum ArrayType
{
	NoCash			= 0,	//Has no cash
	ConstantCash ,			//Allocate specified size
	ExpandCash ,			//Allocate if need. Don't free.
};

///////////////////////////////////////////////////////////////////////////
// classes define

/****************************************************************************
ArrayBase 
****************************************************************************/
/*===========================================================================
[ class purpose ]
===========================================================================*/
/////////////////////////////////////////////////////////////////////////////
template< class TEMP >
class ArrayBase
{
	real_copy_impossible( ArrayBase );
		
// variable
private:
	TEMP	*pData;
	int		Num;

// private functions 
private:
/*===========================================================================
Free data.
                                               
@retval			---
---------------------------------------------------------------------------*/
void Free()
{
	if( pData != 0 )
		delete	[]pData;
	pData	= 0;
	Num		= 0;
}
// protected functions
protected:
/*===========================================================================
Resize array. However before data is hold.

@retval			---
---------------------------------------------------------------------------*/
void ResizeHoldwithNum
		(
		int		num , 			//!< [in] new array size
		int		holdnum			//!< [in] hold data num
		)
{
	// no change
	if( num == Num )
		return;
		
	// allocate memory
	TEMP	*pnew;
	if( num != 0 )
		pnew = new TEMP[ num ];
	else
		pnew = 0;	
	real_assert( !( pnew == 0 && num != 0 ) , "Array1:Fail to allocate memory." );

	// write new data
	int		off;
	holdnum	= holdnum < num ? holdnum : num;
	for ( off = 0 ; off < holdnum ; off++ )
		pnew[ off ]	= pData[ off ];

	// free memory
	if( pData != 0 )
		delete []pData;

	// set param
	pData	= pnew;
	Num		= num;
}
// public functions 
public:
/*===========================================================================
construct
                                               
@retval			---
---------------------------------------------------------------------------*/
ArrayBase() : pData( 0 ) , Num( 0 )
{
}
/*===========================================================================
construct

@retval			---
---------------------------------------------------------------------------*/
ArrayBase
		(
		int		num
		)
{
	if( num != 0 )
		pData	= new TEMP[ num ];
	else
		pData	= 0;
	real_assert( !( pData == 0 && num != 0 ) , "Array1:fail to allocate memory" );
	Num = num;
}
/*===========================================================================
destruct
                                               
@retval			---
---------------------------------------------------------------------------*/
virtual
~ArrayBase()
{
	Free();
}
/*===========================================================================
reference operator

@retval			---
---------------------------------------------------------------------------*/
TEMP& operator[]
		(
		int		n
		)
{
	real_assert( !( n < 0 || n >= Num ) , "Array1:access is out of data" );

	return pData[ n ];
}
/*===========================================================================
reference operator

@retval			---
---------------------------------------------------------------------------*/
const TEMP& operator[]
		(
		int		n
		)const
{
	real_assert( !( n < 0 || n >= Num ) , "Array1:access is out of data" );

	return pData[ n ];
}
/*===========================================================================
GetPtr
                                               
@retval			---
---------------------------------------------------------------------------*/
TEMP* GetPtr()const
{
	return pData;
}
/*===========================================================================
Get const pointer.
                                               
@retval			---
---------------------------------------------------------------------------*/
const TEMP* GetConstPtr()const
{
	return pData;
}
/*===========================================================================
Resize array. However data is break.

@retval			---
---------------------------------------------------------------------------*/
void Resize
		(
		int		num		//!< [in]  new array size
		)
{
	//if no change , return
	if( num == Num )
		return;
		
	// free memory.
	Free();

	// allocate memory
	if( num != 0 )
		pData = new TEMP[ num ];
	else
		pData = 0;	
	Num = num;

	real_assert( !( pData == 0 && num != 0 ) , "Array1:fail to allocate memory" );
}
/*===========================================================================
resize array. however data is break.

@retval			---
---------------------------------------------------------------------------*/
void Resize
		(
		int		num , 		//!< [in]  new array size
		TEMP	&init		//!< [in]  initialized data
		)
{
	if( num == Num )
		return;
		
	Free();

	// allocate memory.
	if( num != 0 )
		pData = new TEMP[ num ];
	else
		pData = 0;	

	// initialize allocate data
	int		off;
	for( off = 0 ;off < num ; off++ )
		pData[ off ]	= init;
	Num = num;

	real_assert( !( pData == 0 && num != 0 ) , "Array1:fail to allocate memory" );
}
/*===========================================================================
resize array. however data is hold

@retval			---
---------------------------------------------------------------------------*/
void ResizeHold
		(
		int		num ,		//!< [in]  new array size
		TEMP	&init		//!< [in]  if array is larger , data is initialized with this.
		)
{
	// check no change
	if( num == Num )
		return;
		
	// allocate memory
	TEMP	*pnew;
	if( num != 0 )
		pnew = new TEMP[ num ];
	else
		pnew = 0;	
	real_assert( !( pData == 0 && num != 0 ) , "Array1:fail to allocate memory" );

	// set data
	int		off;
	if ( Num > num )
	{
		for ( off = 0 ; off < num ; off++ )
			pnew[ off ]	= pData[ off ];
	}
	else
	{
		for ( off = 0 ; off < Num ; off++ )
			pnew[ off ]	= pData[ off ];
		for (  ; off < num ; off++ )
			pnew[ off ]	= init;
	}
	// free memory
	if( pData != 0 )
		delete []pData;

	// set parameter
	pData	= pnew;
	Num		= num;
}
/*===========================================================================
resize array. however data is hold

@retval			---
---------------------------------------------------------------------------*/
void ResizeHold
		(
		int		num			//!< [in] new array size
		)
{
	//check change
	if( num == Num )
		return;
		
	//allocate memory
	TEMP	*pnew;
	if( num != 0 )
		pnew = new TEMP[ num ];
	else
		pnew = 0;	
	real_assert( !( pnew == 0 && num != 0 ) , "Array1:fail to allocate memory" );

	//set data
	int		off;
	if ( Num > num )
	{
		for ( off = 0 ; off < num ; off++ )
			pnew[ off ]	= pData[ off ];
	}
	else
	{
		for ( off = 0 ; off < Num ; off++ )
			pnew[ off ]	= pData[ off ];
	}
	//free memory
	if( pData != 0 )
		delete []pData;

	// set parameter
	pData	= pnew;
	Num		= num;
}
/*===========================================================================
if array is smaller , allocate array.
                                               
@retval			---
---------------------------------------------------------------------------*/
void ResizeIfSmall
		(
		int		num		//!< [in]  new size of array
		)
{
	int		s_num	= GetDataNum();
	if( s_num >= num )
		return;
	Resize( num );
}
/*===========================================================================
add data at back

@retval			( offset of added data )
---------------------------------------------------------------------------*/
int AddData
		(
		TEMP	&data		//!< [in]  additional data
		)
{
	ResizeHold( Num + 1 , data );
	return Num - 1;
}
/*===========================================================================
add data at back

@retval			( offset of additional data )
---------------------------------------------------------------------------*/
int AddData()
{
	ResizeHold( Num + 1 );
	return Num - 1;
}
/*===========================================================================
insert data

@retval			---
---------------------------------------------------------------------------*/
void InsertData
		(
		int			setoff,		//!< [in]  position for insert
		const TEMP	&data		//!< [in]  additional data
		)
{
	real_assert( !( setoff < 0 || setoff > Num ) , "Array1:insert pos is out of data." );
	
	//allocate memory
	TEMP	*pnew = new TEMP[ Num + 1 ];
	real_assert( !( pnew == 0 ) , "Array1:fail to allocate memory." );

	//set data
	int		t_off , s_off;
	for ( t_off = 0 , s_off = 0 ; s_off < setoff ; t_off++ , s_off++ )
		pnew[ t_off ]	= pData[ s_off ];

	pnew[ t_off ]	= data;
	t_off++;
	
	for (  ; s_off < Num ; s_off++ , t_off++ )
		pnew[ t_off ]	= pData[ s_off ];

	//free memory
	if( pData != 0 )
		delete []pData;

	// set parameter
	pData	= pnew;
	Num		= Num + 1;
}
/*===========================================================================
delete data.

@retval			---
---------------------------------------------------------------------------*/
void DeleteData
		(
		int		deloff		//!< [in]  position of data to delete.
		)
{
	real_assert( !( deloff < 0 || deloff >= Num ) , "Array1:  position to delete is out of data" );

	//allocate memory
	TEMP	*pnew;
	if ( Num - 1 != 0 )
	{
		pnew 	= new TEMP[ Num - 1 ];
		real_assert( pnew != 0 , "Array1:fail to allocate memory" );
	}
	else	
		pnew	= 0;	

	//set data
	int		t_off , s_off;
	for ( t_off = 0 , s_off = 0 ; s_off < deloff ; t_off++ , s_off++ )
		pnew[ t_off ]	= pData[ s_off ];

	s_off++;
	
	for (  ; s_off < Num ; s_off++ , t_off++ )
		pnew[ t_off ]	= pData[ s_off ];

	//free memory
	if( pData != 0 )
		delete []pData;

	//set parameter
	pData	= pnew;
	Num		= Num - 1;
}
/*===========================================================================
acquire num of data.

@retval			( num of data )
---------------------------------------------------------------------------*/
int GetDataNum()const 
{
	return Num;
}
/*===========================================================================
serch( need TEMP == operator )
                                               
@retval			( data position. if not find,return -1 )
---------------------------------------------------------------------------*/
int Search
		(
		const TEMP	&data ,			//!< [in] data to search
		int			max				//!< [in] maximum num to search. if -1 , search all.
		)const
{
	int		off , num	= max == -1 ? Num : max;
	num	= num > Num ? Num : num;
	
	for( off = 0 ; off < num ; off++ )
	{
		if( pData[ off ] == data )
			return off;
	}
	return -1;
}
};
/****************************************************************************
Array 
****************************************************************************/
/*===========================================================================
[ class purpose ]

===========================================================================*/
/////////////////////////////////////////////////////////////////////////////
template< class TEMP >
class Array : protected ArrayBase< TEMP >
{
// variable
private:
	ArrayType		Type;
	int				Num;
	int				ExtraNum;
	
// private functions 
private:
/*===========================================================================
Acquire size to allocate actually.
                                               
@retval			---
---------------------------------------------------------------------------*/
int GetCashNum
		(
		int		num
		)
{
	if( Type == NoCash )
		return num;
		
	int		c_num = ArrayBase< TEMP >::GetDataNum();
	if( Type == ExpandCash )
	{
		if( num > c_num )
			return num + ExtraNum;
		else
			return c_num;
	}
	else if( Type == ConstantCash )
	{
		if( num > c_num )
			return num + ExtraNum;
		if( num == c_num )
			return c_num;
		if( num < c_num )
		{
			if( num <= c_num - ExtraNum * 2 )
				return num + ExtraNum;
			else
				return c_num;
		}
	}
	real_assert( false , "Array1:UnknownType" );
	return num;
}
// protected functions
protected:
// public functions 
public:
/*===========================================================================
construct
                                               
@retval			---
---------------------------------------------------------------------------*/
Array() : Type( NoCash ) , Num( 0 ) , ExtraNum( 0 ) 
{
}
/*===========================================================================
construct
                                               
@retval			---
---------------------------------------------------------------------------*/
Array
		(
		int		num
		) : Type( NoCash ) , Num( 0 ) ,ExtraNum( 0 )  
{
	Resize( num );
}
/*===========================================================================
construct
                                               
@retval			---
---------------------------------------------------------------------------*/
Array
		(
		ArrayType	type , 
		int			extranum
		) : Type( type ) , Num( 0 ) , ExtraNum( extranum ) 
{
}
/*===========================================================================
construct
                                               
@retval			---
---------------------------------------------------------------------------*/
Array
		(
		ArrayType	type , 
		int			extranum , 
		int			num
		) : Type( type ) , ExtraNum( extranum ) , Num( 0 )
{
	Resize( num );
}
/*===========================================================================
destruct
                                               
@retval			---
---------------------------------------------------------------------------*/
~Array()
{
}
/*===========================================================================
reference operator

@retval			---
---------------------------------------------------------------------------*/
TEMP& operator[]
		(
		int		n
		)
{
	real_assert( !( n < 0 || n >= Num ) , "Array1:access is out of data" );
	return ( *( ArrayBase< TEMP >* )this )[ n ];
}
/*===========================================================================
reference operator

@retval			---
---------------------------------------------------------------------------*/
const TEMP& operator[]
		(
		int		n
		)const
{
	real_assert( !( n < 0 || n >= Num ) , "Array1:access is out of data" );
	return ( *( ArrayBase< TEMP >* )this )[ n ];
}
/*===========================================================================
set cash size.
                                               
@retval			---
---------------------------------------------------------------------------*/
void SetExtraNum
		(
		int		extra
		)
{
	ExtraNum	= extra;
}
/*===========================================================================
set type of cash.
                                               
@retval			---
---------------------------------------------------------------------------*/
void SetType
		(
		ArrayType		type
		)
{
	Type	= type;
}
/*===========================================================================
get pointer.
                                               
@retval			---
---------------------------------------------------------------------------*/
TEMP* GetPtr()const
{
	return ArrayBase< TEMP >::GetPtr();
}
/*===========================================================================
get constant pointer.
                                               
@retval			---
---------------------------------------------------------------------------*/
const TEMP* GetConstPtr()const
{
	return ArrayBase< TEMP >::GetConstPtr();
}
/*===========================================================================
resize array. however data is break.

@retval			---
---------------------------------------------------------------------------*/
void Resize
		(
		int		num		//!< [in] new size
		)
{
	ArrayBase< TEMP >::Resize( GetCashNum( num ) );
	Num	= num;
}
/*===========================================================================
resize array. however data is hold.

@retval			---
---------------------------------------------------------------------------*/
void ResizeHold
		(
		int		num 		//!< [in] new size of array
		)
{
	ArrayBase< TEMP >::ResizeHoldwithNum( GetCashNum( num ) , Num );
	Num	= num;
}
/*===========================================================================
resize array. however data is hold.

@retval			---
---------------------------------------------------------------------------*/
void ResizeHold
		(
		int		num , 		//!< [in] new size of array.
		TEMP	&init		//!< [in] initialize to new data.
		)
{
	ArrayBase< TEMP >::ResizeHoldwithNum( GetCashNum( num ) , Num );
	
	int		off;
	TEMP	*pdata	= ArrayBase< TEMP >::GetPtr();
	for( off = Num ; off < num ; off++ )
		pdata[ off ]	= init;
	Num	= num;
}
/*===========================================================================
resize array. however data is hold.

@retval			---
---------------------------------------------------------------------------*/
void ResizeHold
		(
		int			num , 		//!< [in] new size of array.
		const TEMP	&init		//!< [in] initialize data of new
		)
{
	ArrayBase< TEMP >::ResizeHoldwithNum( GetCashNum( num ) , Num );
	
	int		off;
	TEMP	*pdata	= ArrayBase< TEMP >::GetPtr();
	for( off = Num ; off < num ; off++ )
		pdata[ off ]	= init;
	Num	= num;
}
/*===========================================================================
add data at back.

@retval			( offset of additional data )
---------------------------------------------------------------------------*/
int AddData
		(
		TEMP	&data		//!< [in]  data to add
		)
{
	ResizeHold( Num + 1 , data );
	return Num - 1;
}
/*===========================================================================
add data at back

@retval			( offset of additional data. )
---------------------------------------------------------------------------*/
int AddData
		(
		const TEMP	&data		//!< [in] data to add
		)
{
	ResizeHold( Num + 1 , data );
	return Num - 1;
}
/*===========================================================================
add data at back.

@retval			( offset of additional data. )
---------------------------------------------------------------------------*/
int AddData()
{
	ResizeHold( Num + 1 );
	return Num - 1;
}
/*===========================================================================
add data at back

@retval			( offset of additional data. )
---------------------------------------------------------------------------*/
int AddData
		(
		TEMP		*pdata , 	//!< [in] data to add
		int			num			//!< [in] num of data to add
		)
{
	int32		oldnum = Num;
	ResizeHold( Num + num );
	
	TEMP	*pardata	= ArrayBase< TEMP >::GetPtr();
	pardata += oldnum;
	int32	off;
	for( off = 0 ; off < num ; off++ )
		pardata[ off ] = pdata[ off ];
	
	return oldnum;
}
/*===========================================================================
insert data.

@retval			---
---------------------------------------------------------------------------*/
void InsertData
		(
		int			setoff,		//!< [in]  offset to insert.
		const TEMP	&data		//!< [in]  data to add
		)
{
	real_assert( !( setoff < 0 || setoff > Num ) , "Array1: insert position is out of data." );
	
	ResizeHold( Num + 1 );
	int		off;
	TEMP	*pdata	= ArrayBase< TEMP >::GetPtr();
	for( off = Num - 1 ; off > setoff ; off-- )
	{
		pdata[ off ]	= pdata[ off - 1 ];
	}
	pdata[ off ]	= data;
}
/*===========================================================================
insert data.

@retval			---
---------------------------------------------------------------------------*/
void InsertData
		(
		int			setoff		//!< [in] offset to insert.
		)
{
	real_assert( !( setoff < 0 || setoff > Num ) , "Array1:insert position is out of data." );
	
	ResizeHold( Num + 1 );
	int		off;
	TEMP	*pdata	= ArrayBase< TEMP >::GetPtr();
	for( off = Num - 1 ; off > setoff ; off-- )
	{
		pdata[ off ]	= pdata[ off - 1 ];
	}
}
/*===========================================================================
delete data.

@retval			---
---------------------------------------------------------------------------*/
void DeleteData
		(
		int		deloff		//!< [in] position of data to delete.
		)
{
	real_assert( !( deloff < 0 || deloff >= Num ) , "Array1:position to delete is out of data." );


	int		off;
	TEMP	*pdata	= ArrayBase< TEMP >::GetPtr();
	for( off = deloff ; off < Num - 1 ; off++ )
	{
		pdata[ off ]	= pdata[ off + 1 ];
	}
	Num--;
}
/*===========================================================================
delete data.

@retval			---
---------------------------------------------------------------------------*/
void DeleteData
		(
		int		deloff , 		//!< [in] position of data to delete.
		int		delnum			//!< [in] number of delete.
		)
{
	int		start	= deloff;
	int		end		= deloff + delnum;
	if( start < 0 )
		start = 0;
	if( end > GetDataNum() )
		end = GetDataNum();
	int		len	= end - start;
	if( len < 0 )
		return;
	
	int		off;
	TEMP	*pdata	= ArrayBase< TEMP >::GetPtr();
	int		tgt = start , src = start + len;
	for( off = start + len ; off < Num ; off++ )
	{
		pdata[ tgt ]	= pdata[ src ];
		tgt++;
		src++;
	}
	ResizeHold( Num - len );
}
/*===========================================================================
aquire num of data.

@retval			( num of data )
---------------------------------------------------------------------------*/
int GetDataNum()const 
{
	return Num;
}
/*===========================================================================
serch data.( TEMP class need == operator. )
                                               
@retval			( position of data to find. if not find,return -1. )
---------------------------------------------------------------------------*/
int Search
		(
		const TEMP	&data			//!< [in] data to search.
		)const
{
	return ArrayBase< TEMP >::Search( data , Num );
}
//=======================================================================
//!	check existance of data.
//!	@retval			---
//-----------------------------------------------------------------------
bool IsDataExist
		(
		int		off
		)
{
	if( off < 0 )
		return false;
	if( GetDataNum() <= off )
		return false;
	return true;
}


};

///////////////////////////////////////////////////////////////////////////
// global variable define

///////////////////////////////////////////////////////////////////////////
// global functions define

};	//namespace Array1

//using namespace Array1;		

#endif

