/****************************************************************************/
/*!
   	@file		Event1_pub.h
	@author 	Fanzo
 	@date 		2007/10/28
*/
/****************************************************************************/

#ifndef		Event1_pub_h
#define		Event1_pub_h

///////////////////////////////////////////////////////////////////////////
//include files
#include	"Real.h"
#ifdef __linux__
	#include	<pthread.h>
#endif
namespace Thread1
{
using namespace real;
///////////////////////////////////////////////////////////////////////////
// preprocessor deifne

///////////////////////////////////////////////////////////////////////////
// type define

///////////////////////////////////////////////////////////////////////////
// classes define

/****************************************************************************
"Event" class 
****************************************************************************/
/*===========================================================================
//!	
===========================================================================*/
/////////////////////////////////////////////////////////////////////////////
class Event
{
	real_copy_impossible( Event );
	
// variable member
private:
#ifdef WIN32
	HANDLE		m_Event;
#elif __APPLE__
	MPEventID	m_Event;
#elif __linux__
	pthread_cond_t	m_Condition;
	pthread_mutex_t	m_Mutex;
	bool			m_Event;
#endif
// private functions
private:
// protect functions
protected:
// public functions
public:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
Event() 
#ifdef __linux__
	: m_Event( false )
#endif
{
#ifdef WIN32
	m_Event = CreateEvent( NULL, FALSE, FALSE, NULL );
	real_assert( m_Event != NULL , "CreateEvent failed." );
#elif __APPLE__
	real_verify( noErr == MPCreateEvent( &m_Event ) );
#elif __linux__
	real_verify( 0 == pthread_mutex_init( &m_Mutex , NULL ) );
	real_verify( 0 == pthread_cond_init( &m_Condition , NULL ) );
#endif
}
//=======================================================================
//!	destruct
//-----------------------------------------------------------------------
~Event()
{
#ifdef WIN32
	real_verify( TRUE == CloseHandle( m_Event ) );
#elif __APPLE__
	real_verify( noErr == MPDeleteEvent( m_Event ) );
#elif __linux__
	real_verify( 0 == pthread_cond_destroy( &m_Condition ) );
	real_verify( 0 == pthread_mutex_destroy( &m_Mutex ) );
#endif
}
//=======================================================================
//!	set event
//!	@retval			---
//-----------------------------------------------------------------------
inline
void SetEvent()
{
#ifdef WIN32
	real_verify( TRUE == ::SetEvent( m_Event ) );
#elif __APPLE__
	real_verify( noErr == MPSetEvent( m_Event , 1 ) );
#elif __linux__
	real_verify( 0 == pthread_mutex_lock( &m_Mutex ) );
	m_Event	= true;
	real_verify( 0 == pthread_cond_signal( &m_Condition ) );
	real_verify( 0 == pthread_mutex_unlock( &m_Mutex ) );
#endif
}
//=======================================================================
//!	wait
//!	@retval			---
//-----------------------------------------------------------------------
inline
bool Wait()
{
#ifdef WIN32
	if( WAIT_OBJECT_0 != ::WaitForSingleObject( m_Event , INFINITE ) )
		return false;
	return true;
#elif __APPLE__
	if( noErr != MPWaitForEvent( m_Event , NULL , kDurationForever ) )
		return false;
	return true;
#elif __linux__
	real_verify( 0 == pthread_mutex_lock( &m_Mutex ) );
	while( m_Event == false )
	{
		if( 0 != pthread_cond_wait( &m_Condition , &m_Mutex ) )
		{
			if( m_Event == false )
			{	
				real_verify( 0 == pthread_mutex_unlock( &m_Mutex ) );
				return false;
			}
		}
	}
	m_Event	= false;
	real_verify( 0 == pthread_mutex_unlock( &m_Mutex ) );
	return true;
#endif
}
//=======================================================================
//!	wait timeout
//!	@retval			if timeout , return false
//-----------------------------------------------------------------------
inline
bool WaitTimeout
		(
		uint32		msec
		)
{
#ifdef WIN32
	if( WAIT_TIMEOUT == ::WaitForSingleObject( m_Event , msec ) )
		return false;
	return true;
#elif __APPLE__
	if( kMPTimeoutErr == MPWaitForEvent( m_Event , NULL , msec * kDurationMillisecond ) )
		return false;
	return true;
#elif __linux__
	timespec to;
	to.tv_sec	= time(NULL) + msec / 1000;
	to.tv_nsec 	= ( msec % 1000 ) * 1000;

	real_verify( 0 == pthread_mutex_lock( &m_Mutex ) );
	while( m_Event == false )
	{
		if( 0 != pthread_cond_timedwait( &m_Condition , &m_Mutex , &to ) )
		{
			real_verify( 0 == pthread_mutex_unlock( &m_Mutex ) );
			return false;
		}
	}
	m_Event	= false;
	real_verify( 0 == pthread_mutex_unlock( &m_Mutex ) );

	return true;
#endif
}
};
///////////////////////////////////////////////////////////////////////////
// global variable define

///////////////////////////////////////////////////////////////////////////
// global functions define

};	//namespace Event1

//using namespace Event1;		

#endif

