/****************************************************************************/
/*!
   	@file		Object.h
	@author 	Fanzo
 	@date 		2007/10/17
*/
/****************************************************************************/

#ifndef		real_Object_h
#define		real_Object_h

///////////////////////////////////////////////////////////////////////////
//include files
#ifdef WIN32
	#include	<windows.h>
#elif __linux__
	#include	<stdlib.h>
	#include	<pthread.h>
#endif
#include <stddef.h>

#pragma pack( push , 1 )		//set align

namespace real
{

struct alloc_info
{
	void					*m_Address;
	void					*m_EndAddress;
	gc_info					*m_GCInfo;
	atomic_ptr< gc_ptr >	*m_PrevListPtr;
};



#ifdef WIN32
inline 
alloc_info* GetTlsAllocInfoPtr()
{
	return ( alloc_info* )TlsGetValue( __real_TlsIndex );
}
inline 
void SetTlsAllocInfoPtr(alloc_info*ptr)
{
	real_verify( TRUE == TlsSetValue( __real_TlsIndex , ( LPVOID )ptr ) );
}
#elif __APPLE__
inline 
alloc_info* GetTlsAllocInfoPtr()
{
	return ( alloc_info* )MPGetTaskStorageValue ( __real_TlsIndex );
}
inline 
void SetTlsAllocInfoPtr(alloc_info*ptr)
{
	real_verify( noErr == MPSetTaskStorageValue ( __real_TlsIndex , ( TaskStorageValue )ptr ) );
}
#elif __linux__
inline
alloc_info* GetTlsAllocInfoPtr()
{
	return ( alloc_info* )pthread_getspecific( __real_TlsIndex ); ;
}
inline 
void SetTlsAllocInfoPtr(alloc_info*ptr)
{
	real_verify( 0 == pthread_setspecific( __real_TlsIndex , ( const void * )ptr ) );
}
#endif
///////////////////////////////////////////////////////////////////////////
// preprocessor deifne

#define		query_begin()											\
protected:															\
void* Query_oride													\
		(															\
		const real::guid &riid										\
		)															\
{																	

#define		interface_hook( INTER , INTER_GUID )			\
	if( riid == INTER_GUID::get_const() )						\
		return static_cast< INTER* >( this );				

#define		query_end( SUPER_CLASS )								\
	return SUPER_CLASS::Query_oride( riid );						\
}

///////////////////////////////////////////////////////////////////////////
// type define

///////////////////////////////////////////////////////////////////////////
// classes define

class gc_ary;
class gc_obj;
class gc_obj_base;

template<class INTER,class INTER_GUID>
class iary_element;
template<class INTER , class INTER_GUID>
class i_obj;
template<class INTER , class INTER_GUID>
class i_ary;

struct header
{
	gc_object_info	m_GCInfo;
	gc_obj_base*	m_BasePtr;
};

/****************************************************************************
"gc_obj_base" class 
****************************************************************************/
class gc_obj_base
{
	real_copy_impossible( gc_obj_base );

// variable member
private:
	gc_info			*m_ThisGCInfo;

//=======================================================================
//!	delete gc function
//-----------------------------------------------------------------------
static
void real_call DeleteGC
		(
		gc_info* info, 
		void*
		)
{
	header	*pgc = ( header* )info;
	delete pgc->m_BasePtr;
}

// new & delete override functions
public:
//=======================================================================
//!	new override function
//-----------------------------------------------------------------------
static 
void *operator new
		(
		size_t		size
		)
{
	// allocate
	uint8		*p		= ( uint8* )malloc( size + sizeof( header ) );
	header		*pgc	= ( header* )p;
	
	// set gc_info
	pgc->m_GCInfo.m_GCInfo.m_Size			= ( uint32 )( size + sizeof( header ) );
	pgc->m_GCInfo.m_GCInfo.m_Trace			= true;
	pgc->m_GCInfo.m_GCInfo.m_Next			= 0;
	pgc->m_GCInfo.m_GCInfo.m_DeleteGC		= DeleteGC;
	pgc->m_GCInfo.m_GCInfo.m_DeleteGCParam	= 0;
	pgc->m_GCInfo.m_GCInfo.m_AllocType		= Object_AllocType;
	pgc->m_GCInfo.m_GCInfo.m_State.initialize( 0x80000001 );
	pgc->m_GCInfo.m_NextPtr.initialize( 0 );
	pgc->m_BasePtr							= 0;

	// set thread local info
	alloc_info	*infoptr	= GetTlsAllocInfoPtr();
	infoptr->m_Address		= p;
	infoptr->m_EndAddress	= ( void* )( ( uint8* )p + size + sizeof( header ) );
	infoptr->m_GCInfo		= &pgc->m_GCInfo.m_GCInfo;
	infoptr->m_PrevListPtr	= &pgc->m_GCInfo.m_NextPtr;
	
	//regist
	RegistRealGC( &pgc->m_GCInfo.m_GCInfo );

	return p + sizeof( header );
}
private:
//=======================================================================
//!	new[] override function
//-----------------------------------------------------------------------
static 
void* operator new[]
		(
		size_t size
		) throw()
{
	real_assert( false , "object_base can't allocate array." );
	return 0;
}
public:
//=======================================================================
//!	delete override function
//-----------------------------------------------------------------------
static 
void operator delete
		(
		void		*p
		)
{
	free( ( ( uint8* )p ) - sizeof( header ) );
}
private:
//=======================================================================
//!	delete override function
//-----------------------------------------------------------------------
static 
void operator delete[]
		(
		void *p
		)
{
	real_assert( false , "object_base can't allocate array." );
}

// override functions
public:
/*===========================================================================
Query
                                               
@retval			---
---------------------------------------------------------------------------*/
virtual
void* Query_oride
		(
		const guid &riid
		)
{
	return 0;
}

// protect functions
protected:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
gc_obj_base()
{
	alloc_info	*infoptr = GetTlsAllocInfoPtr();
	real_assert( infoptr != 0 , "__tls_AllocInfoPtr is invalid." );
	m_ThisGCInfo	= infoptr->m_GCInfo;
	( ( header* )m_ThisGCInfo )->m_BasePtr	= this;
}
//=======================================================================
//!	destruct
//-----------------------------------------------------------------------
virtual
~gc_obj_base()
{
}
//=======================================================================
//!	this object
//-----------------------------------------------------------------------
inline
gc_obj ThisObject()const;

// public functions
public:
//=======================================================================
//!	Query
//-----------------------------------------------------------------------
void* Query
		(
		const guid &riid
		)
{
	return Query_oride( riid );
}
};


/****************************************************************************
"ary_element" class 
****************************************************************************/
class ary_element
{
	friend gc_ary new_array(uint32);
	friend class gc_obj;
	
	template<class INTER , class INTER_GUID>
	friend class i_obj;

	template<class INTER , class INTER_GUID>
	friend class iary_element;
	
// variable member
protected:
	gc_element_ptr		m_Ptr;

//=======================================================================
//!	delete gc function
//-----------------------------------------------------------------------
static
void real_call DeleteGC
		(
		gc_info* info, 
		void*
		)
{
	delete[] ( ( ary_element* )( ( gc_array_info* )info )->m_pArray );
}

// new & delete override functions
private:
//=======================================================================
//!	new override function
//-----------------------------------------------------------------------
static 
void *operator new
		(
		size_t		size
		) throw()
{
	real_assert( false , "object_base can't allocate array." );
	return 0;
}
public:
//=======================================================================
//!	new[] override function
//-----------------------------------------------------------------------
static 
void* operator new[]
		(
		size_t size
		)
{
	uint8			*p		= ( uint8* )malloc( size + sizeof( gc_array_info ) );
	gc_array_info	*pgc	= ( gc_array_info* )p;
	
	pgc->m_GCInfo.m_Size			= ( uint32 )( size + sizeof( gc_array_info ) );
	pgc->m_GCInfo.m_Trace			= true;
	pgc->m_GCInfo.m_Next			= 0;
	pgc->m_GCInfo.m_DeleteGC		= DeleteGC;
	pgc->m_GCInfo.m_DeleteGCParam	= 0;
	pgc->m_GCInfo.m_AllocType		= Array_AllocType;
	pgc->m_GCInfo.m_State.initialize( 0x80000001 );
	pgc->m_ElementNum				= 0;
	pgc->m_ElementSize				= sizeof( ary_element );
	ary_element		*zero = 0;
	pgc->m_ElementPointerOffset		= ( uint32 )( uint64 )( &zero->m_Ptr );
	pgc->m_pArray					= 0;
	
	alloc_info	*infoptr = GetTlsAllocInfoPtr();
	infoptr->m_Address		= p;
	infoptr->m_EndAddress	= ( void* )( ( uint8* )p + size + sizeof( gc_array_info ) );
	infoptr->m_GCInfo		= &pgc->m_GCInfo;
	infoptr->m_PrevListPtr	= 0;
	
	return p + sizeof( gc_array_info );
}
public:
//=======================================================================
//!	delete override function
//-----------------------------------------------------------------------
static 
void operator delete
		(
		void		*p
		)
{
	real_assert( false , "object_base can't allocate array." );
}
public:
//=======================================================================
//!	delete override function
//-----------------------------------------------------------------------
static 
void operator delete[]
		(
		void *p
		)
{
	free( ( ( uint8* )p ) - sizeof( gc_array_info ) );
}

// protected functions
protected:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
ary_element(){}

// public functions
public:
//=======================================================================
//!	destruct
//-----------------------------------------------------------------------
virtual
~ary_element()
{
}
//=======================================================================
//!	release
//-----------------------------------------------------------------------
inline
void release()
{
	m_Ptr.release();
}
//=======================================================================
//!	Query
//-----------------------------------------------------------------------
inline
void* Query
		(
		const guid &riid
		)const
{
	header	*p = ( header* )m_Ptr.get_ptr();
	if( p == 0 )
		return 0;
	real_assert( p->m_GCInfo.m_GCInfo.m_AllocType == Object_AllocType , "query error" );
	return p->m_BasePtr->Query( riid );
}
//=======================================================================
//!	length
//-----------------------------------------------------------------------
inline
int32 length()const
{
	gc_array_info*	ptr = ( gc_array_info* )m_Ptr.get_ptr();
	if( ptr == 0 )
		return 0;
	if( ptr->m_GCInfo.m_AllocType == Object_AllocType )
		return 0;
	return ptr->m_ElementNum;
}
//=======================================================================
//!	ref operator
//-----------------------------------------------------------------------
inline
ary_element& operator[]
		(
		uint32		n
		)
{
	gc_array_info*	ptr = ( gc_array_info* )m_Ptr.get_ptr();

	real_assert( ptr != 0 , "access out of range." );
	real_assert( ptr->m_GCInfo.m_AllocType == Array_AllocType , "access out of range." );
	real_assert( 0 <= n && n < ptr->m_ElementNum , "access out of range." );
	
	return ( ( ary_element* )ptr->m_pArray )[ n ];
}
//=======================================================================
//!	ref operator
//-----------------------------------------------------------------------
inline
const ary_element& operator[]
		(
		uint32		n
		)const
{
	gc_array_info*	ptr = ( gc_array_info* )m_Ptr.get_ptr();

	real_assert( ptr != 0 , "access out of range." );
	real_assert( ptr->m_GCInfo.m_AllocType == Array_AllocType , "access out of range." );
	real_assert( 0 <= n && n < ptr->m_ElementNum , "access out of range." );
	
	return ( ( ary_element* )ptr->m_pArray )[ n ];
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
inline
ary_element& operator=
		(
		const gc_obj& obj
		);
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
template<class INTER,class INTER_GUID>
inline
ary_element& operator=
		(
		const i_obj<INTER,INTER_GUID>& obj
		)
{
	m_Ptr	= obj.m_Ptr.get_ptr();
	return *this;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
inline
ary_element& operator=
		(
		const ary_element& obj
		)
{
	m_Ptr	= obj.m_Ptr.get_ptr();
	return *this;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
template<class INTER,class INTER_GUID>
inline
ary_element& operator=
		(
		const iary_element<INTER,INTER_GUID>& obj
		);
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		bool	b
		)const
{
	return m_Ptr == b ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		bool	b
		)const
{
	return m_Ptr == b ? false : true;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		const gc_obj	&obj
		)const;
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		const gc_obj	&obj
		)const;
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class INTER , class INTER_GUID>
inline
bool operator==
		(
		const i_obj<INTER , INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class INTER , class INTER_GUID>
inline
bool operator!=
		(
		const i_obj<INTER , INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? false : true;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		const ary_element	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		const ary_element	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? false : true;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class INTER , class INTER_GUID>
inline
bool operator==
		(
		const iary_element<INTER , INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class INTER , class INTER_GUID>
inline
bool operator!=
		(
		const iary_element<INTER , INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? false : true;
}
};

/****************************************************************************
"iary_element" class 
****************************************************************************/
template<class INTER , class INTER_GUID=null_guid>
class iary_element : protected ary_element
{
	friend class gc_obj;
	friend class ary_element;
		
	template<class S_INTER,class S_INTER_GUID>
	friend class i_obj;

	template<class S_INTER,class S_INTER_GUID>
	friend i_ary<S_INTER,S_INTER_GUID> new_iarray( uint32 );
	
	template<class S_INTER,class S_INTER_GUID>
	friend class iary_element;


// variable member
private:
	INTER		*m_Query;

//=======================================================================
//!	delete gc function
//-----------------------------------------------------------------------
static
void real_call DeleteGC
		(
		gc_info* info, 
		void*
		)
{
	delete[] ( ( iary_element<INTER , INTER_GUID>* )( ( gc_array_info* )info )->m_pArray );
}

// new & delete override functions
private:
//=======================================================================
//!	new override function
//-----------------------------------------------------------------------
static 
void *operator new
		(
		size_t		size
		)
{
	real_assert( false , "object_base can't allocate array." );
	return 0;
}
public:
//=======================================================================
//!	new[] override function
//-----------------------------------------------------------------------
static 
void* operator new[]
		(
		size_t size
		)
{
	uint8			*p		= ( uint8* )malloc( size + sizeof( gc_array_info ) );
	gc_array_info	*pgc	= ( gc_array_info* )p;
	
	pgc->m_GCInfo.m_Size			= ( uint32 )( size + sizeof( gc_array_info ) );
	pgc->m_GCInfo.m_Trace			= true;
	pgc->m_GCInfo.m_Next			= 0;
	pgc->m_GCInfo.m_DeleteGC		= DeleteGC;
	pgc->m_GCInfo.m_DeleteGCParam	= 0;
	pgc->m_GCInfo.m_AllocType		= Array_AllocType;
	pgc->m_GCInfo.m_State.initialize( 0x80000001 );
	pgc->m_ElementNum				= 0;
	pgc->m_ElementSize				= sizeof( iary_element<INTER,INTER_GUID> );
	{
		iary_element<INTER,INTER_GUID>*	zero = 0;
		pgc->m_ElementPointerOffset		= ( uint32 )( uint64 )( &( zero->m_Ptr ) );
	}
	pgc->m_pArray					= 0;
	
	alloc_info	*infoptr = GetTlsAllocInfoPtr();
	infoptr->m_Address		= p;
	infoptr->m_EndAddress	= ( void* )( ( uint8* )p + size + sizeof( gc_array_info ) );
	infoptr->m_GCInfo		= &pgc->m_GCInfo;
	infoptr->m_PrevListPtr	= 0;
	
	return p + sizeof( gc_array_info );
}
public:
//=======================================================================
//!	delete override function
//-----------------------------------------------------------------------
static 
void operator delete
		(
		void		*p
		)
{
	real_assert( false , "object_base can't allocate array." );
}
public:
//=======================================================================
//!	delete override function
//-----------------------------------------------------------------------
static 
void operator delete[]
		(
		void *p
		)
{
	free( ( ( uint8* )p ) - sizeof( gc_array_info ) );
}


// protected functions
protected:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
iary_element(){}

// public functions
public:
//=======================================================================
//!	release
//-----------------------------------------------------------------------
void release()
{
	m_Ptr.release();
	m_Query	= 0;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
inline
iary_element<INTER , INTER_GUID>& operator=
		(
		const gc_obj& obj
		);
/*
{
	INTER	*query	= ( INTER* )obj.Query( INTER_GUID::get_const() );
	if( query == 0 )
		release();
	else
	{
		m_Ptr	= obj.m_Ptr.get_ptr();
		m_Query	= query;
	}
	return *this;
}
*/
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
inline
iary_element& operator=
		(
		const i_obj<INTER,INTER_GUID>& obj
		)
{
	m_Ptr	= obj.m_Ptr.get_ptr();
	m_Query	= obj.m_Query;
	
	return *this;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
template<class S_INTER , class S_INTER_GUID>
iary_element& operator=
		(
		const i_obj<S_INTER,S_INTER_GUID>& obj
		)
{
	INTER	*query	= ( INTER* )obj.Query( INTER_GUID::get_const() );
	if( query == 0 )
		release();
	else
	{
		m_Ptr	= obj.m_Ptr.get_ptr();
		m_Query	= query;
	}
	return *this;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
inline
iary_element& operator=
		(
		const ary_element& obj
		)
{
	INTER	*query	= ( INTER* )obj.Query( INTER_GUID::get_const() );
	if( query == 0 )
		release();
	else
	{
		m_Ptr	= obj.m_Ptr.get_ptr();
		m_Query	= query;
	}
	return *this;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
inline
iary_element& operator=
		(
		const iary_element<INTER,INTER_GUID>& obj
		)
{
	m_Ptr	= obj.m_Ptr.get_ptr();
	m_Query	= obj.m_Query;
	
	return *this;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
template<class S_INTER , class S_INTER_GUID>
iary_element& operator=
		(
		const iary_element<S_INTER,S_INTER_GUID>& obj
		)
{
	INTER	*query	= ( INTER* )obj.Query( INTER_GUID::get_const() );
	if( query == 0 )
		release();
	else
	{
		m_Ptr	= obj.m_Ptr.get_ptr();
		m_Query	= query;
	}
	return *this;
}
//=======================================================================
//!	ref operator
//-----------------------------------------------------------------------
inline
gc_temporaly<INTER> operator->()const
{
	real_assert( m_Ptr.get_ptr() != 0 , "zero pointer accessed." );
	real_assert( m_Ptr.get_ptr()->m_AllocType == Object_AllocType , "object is not exist" );
	
	return gc_temporaly<INTER>( m_Ptr.get_ptr() , m_Query );
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		bool	b
		)const
{
	return m_Ptr == b ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		bool	b
		)const
{
	return m_Ptr == b ? false : true;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		const gc_obj	&obj
		)const;
/*
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
*/
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		const gc_obj	&obj
		)const;
/*
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
*/
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class S_INTER , class S_INTER_GUID>
inline
bool operator==
		(
		const i_obj<S_INTER , S_INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class S_INTER , class S_INTER_GUID>
inline
bool operator!=
		(
		const i_obj<S_INTER , S_INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? false : true;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		const ary_element	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		const ary_element	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? false : true;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class S_INTER , class S_INTER_GUID>
inline
bool operator==
		(
		const iary_element<S_INTER , S_INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class S_INTER , class S_INTER_GUID>
inline
bool operator!=
		(
		const iary_element<S_INTER , S_INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? false : true;
}
};

/****************************************************************************
"gc_obj" class 
****************************************************************************/
class gc_obj
{
	friend class ary_element;
	
	template<class INTER , class INTER_GUID>
	friend class i_obj;

	template<class INTER , class INTER_GUID>
	friend class iary_element;
	
// variable member
protected:
	gc_ptr		m_Ptr;
	
// public functions
public:
//=======================================================================
//!	destruct
//-----------------------------------------------------------------------
virtual
~gc_obj()
{
}
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
gc_obj
		(
		gc_info		*ptr
		) : 
		m_Ptr( ptr )
{
	m_Ptr.set_mode( true );
}
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
gc_obj()
{
	alloc_info	*infoptr = GetTlsAllocInfoPtr();
	if( infoptr == 0
	|| this < infoptr->m_Address 
	|| this >= infoptr->m_EndAddress )
	{
		m_Ptr.set_mode( true );
}
	else
	{	
		m_Ptr.set_mode( false );
		infoptr->m_PrevListPtr->atomic_set( &m_Ptr );
		infoptr->m_PrevListPtr	= &m_Ptr.m_Next;
	}
}
//=======================================================================
//!	copy construct
//-----------------------------------------------------------------------
gc_obj
		(
		const gc_obj	&obj
		)
{
	alloc_info	*infoptr = GetTlsAllocInfoPtr();
	if( infoptr != 0
	&& infoptr->m_Address <= this
	&& this < infoptr->m_EndAddress )
	{
		m_Ptr.set_mode( false );
		infoptr->m_PrevListPtr->atomic_set( &m_Ptr );
		infoptr->m_PrevListPtr	= &m_Ptr.m_Next;
	}
	m_Ptr	= obj.m_Ptr;
}
//=======================================================================
//!	copy construct
//-----------------------------------------------------------------------
template<class INTER , class INTER_GUID>
gc_obj
		(
		const i_obj<INTER,INTER_GUID>	&obj
		)
{
	alloc_info	*infoptr = GetTlsAllocInfoPtr();
	if( infoptr != 0
	&& infoptr->m_Address <= this
	&& this < infoptr->m_EndAddress )
	{
		m_Ptr.set_mode( false );
		infoptr->m_PrevListPtr->atomic_set( &m_Ptr );
		infoptr->m_PrevListPtr	= &m_Ptr.m_Next;
	}
	m_Ptr	= obj.m_Ptr;
}
//=======================================================================
//!	copy construct
//-----------------------------------------------------------------------
gc_obj
		(
		const ary_element	&obj
		)
{
	alloc_info	*infoptr = GetTlsAllocInfoPtr();
	if( infoptr != 0
	&& infoptr->m_Address <= this
	&& this < infoptr->m_EndAddress )
	{
		m_Ptr.set_mode( false );
		infoptr->m_PrevListPtr->atomic_set( &m_Ptr );
		infoptr->m_PrevListPtr	= &m_Ptr.m_Next;
	}
	m_Ptr	= obj.m_Ptr.get_ptr();
}
//=======================================================================
//!	copy construct
//-----------------------------------------------------------------------
template<class INTER,class INTER_GUID>
gc_obj
		(
		const iary_element<INTER,INTER_GUID>	&obj
		)
{
	alloc_info	*infoptr = GetTlsAllocInfoPtr();
	if( infoptr != 0
	&& infoptr->m_Address <= this
	&& this < infoptr->m_EndAddress )
	{
		m_Ptr.set_mode( false );
		infoptr->m_PrevListPtr->atomic_set( &m_Ptr );
		infoptr->m_PrevListPtr	= &m_Ptr.m_Next;
	}
	m_Ptr	= obj.m_Ptr.get_ptr();
}
//=======================================================================
//!	release
//-----------------------------------------------------------------------
inline
void release()
{
	m_Ptr.release();
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
inline
gc_obj& operator=
		(
		gc_obj& obj
		)
{
	m_Ptr	= obj.m_Ptr;
	return *this;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
template<class INTER,class INTER_GUID>
inline
gc_obj& operator=
		(
		i_obj<INTER,INTER_GUID>& obj
		)
{
	m_Ptr	= obj.m_Ptr;
	return *this;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
inline
gc_obj& operator=
		(
		ary_element& obj
		)
{
	m_Ptr	= obj.m_Ptr.get_ptr();
	return *this;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
template<class INTER , class INTER_GUID>
inline
gc_obj& operator=
		(
		const iary_element<INTER,INTER_GUID>& obj
		)
{
	m_Ptr	= obj.m_Ptr.get_ptr();
	return *this;
}
//=======================================================================
//!	Query
//-----------------------------------------------------------------------
inline
void* Query
		(
		const guid &riid
		)const
{
	header	*p = ( header* )m_Ptr.get_ptr();
	if( p == 0 )
		return 0;
	return p->m_BasePtr->Query( riid );
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		bool	b
		)const
{
	return m_Ptr == b ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		bool	b
		)const
{
	return m_Ptr == b ? false : true;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		const gc_obj	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		const gc_obj	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class S_INTER , class S_INTER_GUID>
inline
bool operator==
		(
		const i_obj<S_INTER , S_INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class S_INTER , class S_INTER_GUID>
inline
bool operator!=
		(
		const i_obj<S_INTER , S_INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? false : true;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		const ary_element	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		const ary_element	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? false : true;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class S_INTER , class S_INTER_GUID>
inline
bool operator==
		(
		const iary_element<S_INTER , S_INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class S_INTER , class S_INTER_GUID>
inline
bool operator!=
		(
		const iary_element<S_INTER , S_INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? false : true;
}
};

/****************************************************************************
"i_obj" class 
****************************************************************************/
template<class INTER , class INTER_GUID = null_guid>
class i_obj : protected gc_obj
{
	friend class gc_obj;
	friend class ary_element;
	friend class iary_element<INTER,INTER_GUID>;
	template<class S_INTER , class S_INTER_GUID>
	friend class iary_element;
	
	template<class S_INTER , class S_INTER_GUID>
	friend class i_obj;
	
// variable member
private:
	INTER*	m_Query;

// public functions
public:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
i_obj
		(
		gc_info		*ptr , 
		INTER	*t_ptr
		) : gc_obj( ptr ) , m_Query( t_ptr )
{
}
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
i_obj() : m_Query( 0 )
{
}
//=======================================================================
//!	copy construct
//-----------------------------------------------------------------------
i_obj
		(
		const gc_obj	&obj
		) : m_Query( 0 )
{
	INTER	*query	= ( INTER* )obj.Query( INTER_GUID::get_const() );
	if( query != 0 )
	{
		m_Ptr	= obj.m_Ptr;
		m_Query = query;
	}
}
//=======================================================================
//!	copy construct
//-----------------------------------------------------------------------
i_obj
		(
		const i_obj<INTER,INTER_GUID>	&obj
		)
{
	m_Ptr	= obj.m_Ptr;
	m_Query = obj.m_Query;
}
//=======================================================================
//!	copy construct
//-----------------------------------------------------------------------
template< class S_INTER , class S_INTER_GUID >
i_obj
		(
		const i_obj<S_INTER,S_INTER_GUID>	&obj
		) : m_Query( 0 )
{
	INTER	*query	= ( INTER* )obj.Query( INTER_GUID::get_const() );
	if( query != 0 )
	{
		m_Ptr	= obj.m_Ptr;
		m_Query = query;
	}
}
//=======================================================================
//!	copy construct
//-----------------------------------------------------------------------
i_obj
		(
		const ary_element	&obj
		) : m_Query( 0 )
{
	INTER	*query	= ( INTER* )obj.Query( INTER_GUID::get_const() );
	if( query != 0 )
	{
		m_Ptr	= obj.m_Ptr;
		m_Query = query;
	}
}
//=======================================================================
//!	copy construct
//-----------------------------------------------------------------------
i_obj
		(
		const iary_element<INTER,INTER_GUID>	&obj
		)
{
	m_Ptr	= obj.m_Ptr.get_ptr();
	m_Query = obj.m_Query;
}
//=======================================================================
//!	copy construct
//-----------------------------------------------------------------------
template< class S_INTER , class S_INTER_GUID >
i_obj
		(
		const iary_element<S_INTER,S_INTER_GUID>	&obj
		) : m_Query( 0 )
{
	INTER	*query	= ( INTER* )obj.Query( INTER_GUID::get_const() );
	if( query != 0 )
	{
		m_Ptr	= obj.m_Ptr;
		m_Query = query;
	}
}
//=======================================================================
//!	release
//-----------------------------------------------------------------------
void release()
{
	m_Ptr.release();
	m_Query	= 0;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
inline
i_obj<INTER,INTER_GUID>& operator=
		(
		const gc_obj	&obj
		)
{
	INTER	*query	= ( INTER* )obj.Query( INTER_GUID::get_const() );
	if( query == 0 )
		release();
	else
	{
		m_Ptr	= obj.m_Ptr;
		m_Query = query;
	}
	return *this;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
inline
i_obj<INTER,INTER_GUID>& operator=
		(
		i_obj<INTER,INTER_GUID>& obj
		)
{
	m_Ptr	= obj.m_Ptr;
	m_Query = obj.m_Query;
	return *this;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
template<class S_INTER,class S_INTER_GUID>
inline
i_obj<INTER,INTER_GUID>& operator=
		(
		i_obj<S_INTER,S_INTER_GUID>& obj
		)
{
	INTER	*query	= ( INTER* )obj.Query( INTER_GUID::get_const() );
	if( query == 0 )
		release();
	else
	{
		m_Ptr	= obj.m_Ptr;
		m_Query = query;
	}
	return *this;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
inline
i_obj<INTER,INTER_GUID>& operator=
		(
		ary_element& obj
		)
{
	INTER	*query = ( INTER* )obj.Query( INTER_GUID::get_const() );
	if( query == 0 )
		release();
	else
	{
		m_Ptr	= obj.m_Ptr.get_ptr();
		m_Query	= query;
	}
	return *this;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
inline
i_obj<INTER,INTER_GUID>& operator=
		(
		const iary_element<INTER,INTER_GUID>& obj
		)
{
	m_Ptr	= obj.m_Ptr.get_ptr();
	m_Query	= obj.m_Query;
	return *this;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
template<class S_INTER , class S_INTER_GUID>
inline
gc_obj& operator=
		(
		const iary_element<S_INTER,S_INTER_GUID>& obj
		)
{
	INTER	*query = ( INTER* )obj.Query( INTER_GUID::get_const() );
	if( query == 0 )
		release();
	else
	{
		m_Ptr	= obj.m_Ptr.get_ptr();
		m_Query	= query;
	}
	return *this;
}
//=======================================================================
//!	Query
//-----------------------------------------------------------------------
inline
void* Query
		(
		const guid &riid
		)const
{
	return gc_obj::Query( riid );
}
//=======================================================================
//!	ref operator
//-----------------------------------------------------------------------
inline
gc_temporaly<INTER> operator->()const
{
	real_assert( m_Ptr.get_ptr() != 0 , "object is not exist." );
	
	return gc_temporaly<INTER>( m_Ptr.get_ptr() , m_Query );
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		bool	b
		)const
{
	return m_Ptr == b ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		bool	b
		)const
{
	return m_Ptr == b ? false : true;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		const gc_obj	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		const gc_obj	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class S_INTER , class S_INTER_GUID>
inline
bool operator==
		(
		const i_obj<S_INTER , S_INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class S_INTER , class S_INTER_GUID>
inline
bool operator!=
		(
		const i_obj<S_INTER , S_INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? false : true;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		const ary_element	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		const ary_element	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? false : true;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class S_INTER , class S_INTER_GUID>
inline
bool operator==
		(
		const iary_element<S_INTER , S_INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class S_INTER , class S_INTER_GUID>
inline
bool operator!=
		(
		const iary_element<S_INTER , S_INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? false : true;
}
};

/****************************************************************************
"gc_ary" class 
****************************************************************************/
class gc_ary
{
// variable member
protected:
	gc_ptr		m_Ptr;
	
// public functions
public:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
gc_ary()
{
	alloc_info	*infoptr = GetTlsAllocInfoPtr();
	if( infoptr != 0
	&& infoptr->m_Address <= this
	&& this < infoptr->m_EndAddress )
	{
		m_Ptr.set_mode( false );	
		infoptr->m_PrevListPtr->atomic_set( &m_Ptr );
		infoptr->m_PrevListPtr	= &m_Ptr.m_Next;
	}
}
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
gc_ary
		(
		gc_info		*ptr
		) : 
		m_Ptr( ptr )
{
}
//=======================================================================
//!	copy construct
//-----------------------------------------------------------------------
gc_ary
		(
		const gc_ary	&obj
		)
{
	alloc_info	*infoptr = GetTlsAllocInfoPtr();
	if( infoptr != 0
	&& infoptr->m_Address <= this
	&& this < infoptr->m_EndAddress )
	{
		m_Ptr.set_mode( false );
		infoptr->m_PrevListPtr->atomic_set( &m_Ptr );
		infoptr->m_PrevListPtr	= &m_Ptr.m_Next;
	}
	m_Ptr	= obj.m_Ptr;
}
//=======================================================================
//!	release
//-----------------------------------------------------------------------
inline
void release()
{
	m_Ptr.release();
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
inline
gc_ary& operator=
		(
		gc_ary& obj
		)
{
	m_Ptr	= obj.m_Ptr;
	return *this;
}
//=======================================================================
//!	length
//-----------------------------------------------------------------------
inline
int32 length()const
{
	gc_array_info*	ptr = ( gc_array_info* )m_Ptr.get_ptr();
	if( ptr == 0 )
		return 0;
	return ptr->m_ElementNum;
}
//=======================================================================
//!	ref operator
//-----------------------------------------------------------------------
inline
ary_element& operator[]
		(
		uint32		n
		)
{
	gc_array_info*	ptr = ( gc_array_info* )m_Ptr.get_ptr();
	real_assert( 0 <= n && n < ptr->m_ElementNum , "access out of range." );
	
	return ( ( ary_element* )( ptr->m_pArray ) )[ n ];
}
//=======================================================================
//!	ref operator
//-----------------------------------------------------------------------
inline
const ary_element& operator[]
		(
		uint32		n
		)const
{
	gc_array_info*	ptr = ( gc_array_info* )m_Ptr.get_ptr();
	real_assert( 0 <= n && n < ptr->m_ElementNum , "access out of range." );
	
	return ( ( ary_element* )( ptr->m_pArray ) )[ n ];
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		bool	b
		)const
{
	return m_Ptr == b ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		bool	b
		)const
{
	return m_Ptr == b ? false : true;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		const gc_ary	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		const gc_ary	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? false : true;
}
};

/****************************************************************************
"i_ary" class 
****************************************************************************/
template<class INTER,class INTER_GUID=null_guid>
class i_ary : protected gc_ary
{
// variable member
private:
// public functions
public:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
i_ary
		(
		gc_info		*ptr
		) : gc_ary( ptr )
{
}
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
i_ary()
{
}
//=======================================================================
//!	copy construct
//-----------------------------------------------------------------------
i_ary
		(
		const i_ary<INTER,INTER_GUID>	&obj
		) : gc_ary( obj )
{
}
//=======================================================================
//!	release
//-----------------------------------------------------------------------
inline
void release()
{
	m_Ptr.release();
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
inline
i_ary& operator=
		(
		i_ary<INTER,INTER_GUID>& obj
		)
{
	m_Ptr	= obj.m_Ptr;
	return *this;
}
//=======================================================================
//!	length
//-----------------------------------------------------------------------
inline
int32 length()const
{
	gc_array_info*	ptr = ( gc_array_info* )m_Ptr.get_ptr();
	if( ptr == 0 )
		return 0;
	return ptr->m_ElementNum;
}
//=======================================================================
//!	ref operator
//-----------------------------------------------------------------------
inline
iary_element<INTER,INTER_GUID>& operator[]
		(
		uint32		n
		)
{
	gc_array_info*	ptr = ( gc_array_info* )m_Ptr.get_ptr();
	real_assert( 0 <= n && n < ptr->m_ElementNum , "access out of range." );
	
	return ( ( iary_element<INTER,INTER_GUID>* )( ptr->m_pArray ) )[ n ];
}
//=======================================================================
//!	ref operator
//-----------------------------------------------------------------------
inline
const iary_element<INTER,INTER_GUID>& operator[]
		(
		uint32		n
		)const
{
	gc_array_info*	ptr = ( gc_array_info* )m_Ptr.get_ptr();
	real_assert( 0 <= n && n < ptr->m_ElementNum , "access out of range." );
	
	return ( ( iary_element<INTER,INTER_GUID>* )( ptr->m_pArray ) )[ n ];
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		bool	b
		)const
{
	return m_Ptr == b ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		bool	b
		)const
{
	return m_Ptr == b ? false : true;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		const i_ary<INTER , INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		const i_ary<INTER , INTER_GUID>	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? false : true;
}
};
///////////////////////////////////////////////////////////////////////////
// function define

//=======================================================================
//!	this object
//-----------------------------------------------------------------------
inline
gc_obj gc_obj_base::ThisObject()const
{
	m_ThisGCInfo->m_State.atomic_increment();	
	return gc_obj( m_ThisGCInfo );	
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
template<class INTER , class INTER_GUID>
inline
iary_element<INTER,INTER_GUID>& iary_element<INTER,INTER_GUID>::operator=
		(
		const gc_obj& obj
		)
{
	INTER	*query	= ( INTER* )obj.Query( INTER_GUID::get_const() );
	if( query == 0 )
		release();
	else
	{
		m_Ptr	= obj.m_Ptr.get_ptr();
		m_Query	= query;
	}
	return *this;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class INTER , class INTER_GUID>
inline
bool iary_element<INTER , INTER_GUID>::operator==
		(
		const gc_obj	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template<class INTER , class INTER_GUID>
inline
bool iary_element<INTER,INTER_GUID>::operator!=
		(
		const gc_obj	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
inline
ary_element& ary_element::operator=
		(
		const gc_obj& obj
		)
{
	m_Ptr	= obj.m_Ptr.get_ptr();
	return *this;
}
//=======================================================================
//!	substitution
//-----------------------------------------------------------------------
template<class INTER,class INTER_GUID>
ary_element& ary_element::operator=
		(
		const iary_element<INTER,INTER_GUID>& obj
		)
{
	m_Ptr	= obj.m_Ptr.get_ptr();
	return *this;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool ary_element::operator==
		(
		const gc_obj	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool ary_element::operator!=
		(
		const gc_obj	&obj
		)const
{
	return m_Ptr == obj.m_Ptr ? false : true;
}

///////////////////////////////////////////////////////////////////////////
// global variable define
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		( 
		bool			exist , 
		const gc_obj	&obj
		)
{
	return ( obj == exist );
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		( 
		bool			exist , 
		const gc_obj	&obj
		)
{
	return ( obj != exist );
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template< class INTER , class INTER_GUID >
inline
bool operator==
		( 
		bool							exist , 
		const i_obj<INTER , INTER_GUID>	&obj
		)
{
	return ( obj == exist );
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template< class INTER , class INTER_GUID >
inline
bool operator!=
		( 
		bool							exist , 
		const i_obj<INTER , INTER_GUID>	&obj
		)
{
	return ( obj != exist );
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		( 
		bool				exist , 
		const ary_element	&obj
		)
{
	return ( obj == exist );
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		( 
		bool				exist , 
		const ary_element	&obj
		)
{
	return ( obj != exist );
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template< class INTER , class INTER_GUID >
inline
bool operator==
		( 
		bool									exist , 
		const iary_element<INTER , INTER_GUID>	&obj
		)
{
	return ( obj == exist );
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
template< class INTER , class INTER_GUID >
inline
bool operator!=
		( 
		bool									exist , 
		const iary_element<INTER , INTER_GUID>	&obj
		)
{
	return ( obj != exist );
}
///////////////////////////////////////////////////////////////////////////
// global variable define

///////////////////////////////////////////////////////////////////////////
// global functions define

//=======================================================================
//!	new gc object
//-----------------------------------------------------------------------
template<class CLASS_NAME>
gc_obj new_object()
{
	alloc_info	*old	= GetTlsAllocInfoPtr();
	alloc_info	info;
	info.m_GCInfo	= 0;
	{
		SetTlsAllocInfoPtr( &info );
	}
	
	CLASS_NAME	*p = new CLASS_NAME();
	real_assert( p !=0 , "new_object allocate failed." );
	real_unused( p );

	{
		SetTlsAllocInfoPtr( old );
	}
	return gc_obj( info.m_GCInfo );
}
//=======================================================================
//!	new gc object
//-----------------------------------------------------------------------
template<class CLASS_NAME>
i_obj<CLASS_NAME,null_guid> new_iobject()
{
	alloc_info	*old	= GetTlsAllocInfoPtr();
	alloc_info	info;
	info.m_GCInfo = 0;
	{
		SetTlsAllocInfoPtr( &info );
	}
	
	CLASS_NAME	*p = new CLASS_NAME();
	real_assert( p !=0 , "new_object allocate failed." ); 
	real_unused( p );

	{
		SetTlsAllocInfoPtr( old );
	}
	return i_obj<CLASS_NAME,null_guid>( info.m_GCInfo , p );
}
//=======================================================================
//!	new gc array
//-----------------------------------------------------------------------
inline
gc_ary new_array
		(
		uint32		num
		)
{
	if( num == 0 )
		return gc_ary();

	alloc_info	*old	= GetTlsAllocInfoPtr();
	alloc_info	info;
	info.m_GCInfo	= 0;
	{
		SetTlsAllocInfoPtr( &info );
	}

	ary_element	*p = new ary_element[ num ];
	real_assert( p !=0 , "new_array allocate failed." ); 
	( ( gc_array_info* )info.m_GCInfo )->m_pArray				= p;
	( ( gc_array_info* )info.m_GCInfo )->m_ElementNum			= num;

	{
		SetTlsAllocInfoPtr( old );
	}
	//regist
	RegistRealGC( info.m_GCInfo );

	return gc_ary( info.m_GCInfo );
}
//=======================================================================
//!	new gc array
//-----------------------------------------------------------------------
template<class INTER,class INTER_GUID>
i_ary<INTER,INTER_GUID> new_iarray
		(
		uint32		num
		)
{
	if( num == 0 )
		return i_ary<INTER,INTER_GUID>();

	alloc_info	*old	= GetTlsAllocInfoPtr();
	alloc_info	info;
	info.m_GCInfo	= 0;
	{
		SetTlsAllocInfoPtr( &info );
	}

	iary_element<INTER,INTER_GUID>	*p = new iary_element<INTER,INTER_GUID>[ num ];
	real_assert( p !=0 , "new_array allocate failed." ); 
	{
		gc_array_info*	ary_info = ( gc_array_info* )info.m_GCInfo;
		ary_info->m_pArray					= p;
		ary_info->m_ElementNum				= num;
	}
	{
		SetTlsAllocInfoPtr( old );
	}
	//regist
	RegistRealGC( info.m_GCInfo );

	return i_ary<INTER,INTER_GUID>( info.m_GCInfo );
}
//=======================================================================
//!	new gc array
//-----------------------------------------------------------------------
template<class INTER>
i_ary<INTER,null_guid> new_iarray
		(
		uint32		num
		)
{
	return new_iarray<INTER,null_guid>( num );
}




};	//namespace


//using namespace NAMESPACE;		

#pragma pack( pop )			//release align

#endif

