/****************************************************************************/
/*!
   	@file		SharedHandle1_pub.h
	@author 	Fanzo
 	@date 		2007/10/28
*/
/****************************************************************************/

#ifndef		SharedHandle1_pub_h
#define		SharedHandle1_pub_h

///////////////////////////////////////////////////////////////////////////
//include files
#include	"Real.h"

namespace SharedHandle1
{
using namespace real;
///////////////////////////////////////////////////////////////////////////
// preprocessor deifne

///////////////////////////////////////////////////////////////////////////
// type define

///////////////////////////////////////////////////////////////////////////
// classes define

/****************************************************************************
"shared_handle" class 
****************************************************************************/
/*===========================================================================
//!	
===========================================================================*/
/////////////////////////////////////////////////////////////////////////////
template< class T >
class shared_handle
{
// variable member
private:
	T				*m_Ptr;
	atomic_uint32	*m_Counter;

// public functions
public:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
shared_handle() : m_Ptr( 0 ) , m_Counter( 0 )
{
}
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
shared_handle
		(
		T	*ptr
		) : m_Ptr( ptr ) , m_Counter( 0 )
{
	if( m_Ptr != 0 )
	{
		m_Counter	= new atomic_uint32();
		m_Counter->atomic_set( 1 );
	}
}
//=======================================================================
//!	copy construct
//-----------------------------------------------------------------------
shared_handle
		(
		const shared_handle	&obj
		) : m_Ptr( obj.m_Ptr ) , m_Counter( obj.m_Counter )
{
	if( m_Counter != 0 )
	{
		m_Counter->atomic_increment();
	}
}
//=======================================================================
//!	destruct
//-----------------------------------------------------------------------
~shared_handle()
{
	if( m_Counter != 0 )
	{
		if( 0 == m_Counter->atomic_decrement_r() )
		{
			delete m_Ptr;
			delete m_Counter;
		}
	}
}
//=======================================================================
//!	substitution
//!	@retval			---
//-----------------------------------------------------------------------
shared_handle<T>& operator=
		(
		const shared_handle<T>& obj
		)
{
	if( obj.m_Counter != 0 )
		obj.m_Counter->atomic_increment();
	if( m_Counter != 0 )
	{
		if( 0 == m_Counter->atomic_decrement_r() )
		{
			delete m_Ptr;
			delete m_Counter;
		}
	}
	m_Counter	= obj.m_Counter;
	m_Ptr		= obj.m_Ptr;
	
	return *this;
}
//=======================================================================
//!	ref operator
//!	@retval			---
//-----------------------------------------------------------------------
T* operator->()const
{
	real_assert( m_Ptr != 0 , "shared_handle: not allocated" );
	return m_Ptr;
}
//=======================================================================
//!	ref operator
//!	@retval			---
//-----------------------------------------------------------------------
T& operator*()const
{
	real_assert( m_Ptr != 0 , "shared_handle: not allocated" );
	return *m_Ptr;
}
//=======================================================================
//!	is allocated
//!	@retval			---
//-----------------------------------------------------------------------
operator bool()const
{
	return ( m_Ptr != 0 );
}
};
///////////////////////////////////////////////////////////////////////////
// global variable define

///////////////////////////////////////////////////////////////////////////
// global functions define

};	//namespace SharedHandle1

//using namespace SharedHandle1;		

#endif

