/****************************************************************************/
/*!
   	@file		CriticalSection1_pub.h
	@author 	Fanzo
 	@date 		2007/10/23
*/
/****************************************************************************/

#ifndef		CriticalSection1_pub_h
#define		CriticalSection1_pub_h

///////////////////////////////////////////////////////////////////////////
//include files
#include	"Real.h"

#ifdef WIN32
	#include	<windows.h>
#elif __linux__
	#include	<pthread.h>
#endif

namespace Thread1
{
///////////////////////////////////////////////////////////////////////////
// preprocessor deifne

///////////////////////////////////////////////////////////////////////////
// type define

///////////////////////////////////////////////////////////////////////////
// classes define

/****************************************************************************
"CriticalSection" class 
****************************************************************************/
class CriticalSection
{
	real_copy_impossible( CriticalSection );
	
private:
#ifdef WIN32
	CRITICAL_SECTION	m_CriticalSection;
#elif __APPLE__
	MPCriticalRegionID 	m_CriticalRegionID;
#elif __linux__
	pthread_mutex_t		m_Mutex;
#else
	#error	Invalid OS
#endif

public:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
CriticalSection()
{
#ifdef WIN32
	InitializeCriticalSection( &m_CriticalSection );
#elif __APPLE__
	real_verify( noErr == MPCreateCriticalRegion( &m_CriticalRegionID ) );
#elif __linux__
	real_verify( 0 == pthread_mutex_init( &m_Mutex , NULL ) );
#endif
}
//=======================================================================
//!	destruct
//-----------------------------------------------------------------------
~CriticalSection()
{
#ifdef WIN32
	DeleteCriticalSection( &m_CriticalSection );
#elif __APPLE__
	real_verify( noErr == MPDeleteCriticalRegion( m_CriticalRegionID ) );
#elif __linux__
	real_verify( 0 == pthread_mutex_destroy( &m_Mutex ) );
#endif
}
//=======================================================================
//!	lock
//!	@retval			---
//-----------------------------------------------------------------------
inline
void Lock()
{
#ifdef WIN32
	EnterCriticalSection( &m_CriticalSection );
#elif __APPLE__
	real_verify( noErr == MPEnterCriticalRegion( m_CriticalRegionID , kDurationForever  ) );
#elif __linux__
	real_verify( 0 == pthread_mutex_lock( &m_Mutex ) );
#endif
}
//=======================================================================
//!	unlock
//!	@retval			---
//-----------------------------------------------------------------------
inline
void Unlock()
{
#ifdef WIN32
	LeaveCriticalSection( &m_CriticalSection );
#elif __APPLE__
	real_verify( noErr == MPExitCriticalRegion( m_CriticalRegionID ) );
#elif __linux__
	real_verify( 0 == pthread_mutex_unlock( &m_Mutex ) );
#endif
}
};

/****************************************************************************
"Autolock" class 
****************************************************************************/
/*===========================================================================
//!	
===========================================================================*/
/////////////////////////////////////////////////////////////////////////////
class Autolock
{
	real_copy_impossible( Autolock );
	
// variable member
private:
	CriticalSection		&m_CS;
	
// public functions
public:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
Autolock
		(
		CriticalSection	&cs
		) : m_CS( cs )
{
	m_CS.Lock();
}
//=======================================================================
//!	destruct
//-----------------------------------------------------------------------
~Autolock()
{
	m_CS.Unlock();
}
};

///////////////////////////////////////////////////////////////////////////
// global variable define

///////////////////////////////////////////////////////////////////////////
// global functions define

};	//namespace CriticalSection1

//using namespace CriticalSection1;		

#endif

