/****************************************************************************/
/*!
   	@file		Semaphore1_pub.h
	@author 	Fanzo
 	@date 		2007/10/28
*/
/****************************************************************************/

#ifndef		Semaphore1_pub_h
#define		Semaphore1_pub_h

///////////////////////////////////////////////////////////////////////////
//include files
#include	"Real.h"
#ifdef __linux__
	#include	<pthread.h>
	#include	<semaphore.h>
#endif
namespace Thread1
{
using namespace real;
///////////////////////////////////////////////////////////////////////////
// preprocessor deifne

///////////////////////////////////////////////////////////////////////////
// type define

///////////////////////////////////////////////////////////////////////////
// classes define

/****************************************************************************
"Semaphore" class 
****************************************************************************/
/*===========================================================================
//!	
===========================================================================*/
/////////////////////////////////////////////////////////////////////////////
class Semaphore
{
	real_copy_impossible( Semaphore );
// variable member
private:
#ifdef WIN32
	HANDLE			m_Semaphore;
#elif __APPLE__
	MPSemaphoreID	m_Semaphore;
#elif __linux__
	sem_t			m_Semaphore;
#endif
// private functions
private:
// protect functions
protected:
// public functions
public:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
Semaphore
		(
		int32		init , 
		int32		max
		)
{
#ifdef WIN32
	m_Semaphore	= CreateSemaphore( NULL , init , max , NULL );
	real_assert( m_Semaphore != NULL , "CreateSemaphore failed." );
#elif __APPLE__
	real_verify( noErr == MPCreateSemaphore( max , init , &m_Semaphore ) );
#elif __linux__
	real_verify( 0 == sem_init( &m_Semaphore , 0 , init ) );
#endif
}
//=======================================================================
//!	destruct
//-----------------------------------------------------------------------
~Semaphore()
{
#ifdef WIN32
	CloseHandle( m_Semaphore );
#elif __APPLE__
	real_verify( noErr == MPDeleteSemaphore( m_Semaphore ) );
#elif __linux__
	real_verify( 0 == sem_destroy( &m_Semaphore ) );
#endif
}
//=======================================================================
//! wait	
//!	@retval			---
//-----------------------------------------------------------------------
inline
bool Wait()
{
#ifdef WIN32
	if( WAIT_OBJECT_0 != ::WaitForSingleObject( m_Semaphore , INFINITE ) )
		return false;
	return true;
#elif __APPLE__
	if( noErr != MPWaitOnSemaphore( m_Semaphore , kDurationForever ) )
		return false;
	return true;
#elif __linux__
	if( 0 != sem_wait( &m_Semaphore ) )
		return false;
	return true;
#endif
}
//=======================================================================
//! wait timeout
//!	@retval			if timeout , return false
//-----------------------------------------------------------------------
inline
bool WaitTimeout
		(
		uint32	msec
		)
{
#ifdef WIN32
	if( WAIT_OBJECT_0 != ::WaitForSingleObject( m_Semaphore , msec ) )
		return false;
	return true;
#elif __APPLE__
	if( noErr != MPWaitOnSemaphore( m_Semaphore , msec * kDurationMillisecond ) )
		return false;
	return true;
#elif __linux__
	timespec to;
	to.tv_sec	= time(NULL) + msec / 1000;
	to.tv_nsec 	= ( msec % 1000 ) * 1000;
	if( 0 != sem_timedwait( &m_Semaphore , &to ) )
		return false;
	return true;
#endif
}
//=======================================================================
//!	post
//!	@retval			---
//-----------------------------------------------------------------------
inline
void Post()
{
#ifdef WIN32
	::ReleaseSemaphore( m_Semaphore , 1 , NULL );
#elif __APPLE__
	real_verify( noErr == MPSignalSemaphore( m_Semaphore ) );
#elif __linux__
	real_verify( 0 == sem_post( &m_Semaphore ) );
#endif
}
};
///////////////////////////////////////////////////////////////////////////
// global variable define

///////////////////////////////////////////////////////////////////////////
// global functions define

};	//namespace Semaphore1

//using namespace Semaphore1;		

#endif

