/****************************************************************************/
/*!
   	@file		Thread1_pub.h
	@author 	Fanzo
 	@date 		2007/10/23
*/
/****************************************************************************/

#ifndef		Thread1_pub_h
#define		Thread1_pub_h

///////////////////////////////////////////////////////////////////////////
//include files
#if defined(__APPLE__) || defined(__linux__)
#include	<pthread.h>
#include	<unistd.h>
#endif

#include	"real.h"
namespace Thread1
{
using namespace real;
///////////////////////////////////////////////////////////////////////////
// preprocessor deifne

///////////////////////////////////////////////////////////////////////////
// type define
enum Priority
{
	Unknown_Priority = 0 , 
	
	Lowest_Priority , 
	BelowNormal_Priority , 
	Normal_Priority , 
	AboveNormal_Priority , 
	Highest_Priority , 
};
///////////////////////////////////////////////////////////////////////////
// classes define

/****************************************************************************
"Thread" class 
****************************************************************************/
/*===========================================================================
//!	
===========================================================================*/
/////////////////////////////////////////////////////////////////////////////
class Thread
{
	real_copy_impossible( Thread );
	
// variable member
private:
#ifdef WIN32
	HANDLE			m_hThread;
	DWORD			m_ThreadID;
#elif defined(__APPLE__)||defined(__linux__)
	pthread_t		m_hThread;
	atomic_uint32	m_Endflag;
#endif

// static functions
private:
#ifdef WIN32
//=======================================================================
//!	thread callback
//!	@retval			---
//-----------------------------------------------------------------------
static
DWORD WINAPI Thread::ThreadFuncStatic
		(
		LPDWORD		lparam
		)
{
	if( lparam != 0 )
	{
		( ( Thread* )lparam )->ThreadProc_oride();
	}
	return 0;
}
#elif defined(__APPLE__) || defined(__linux__)
//=======================================================================
//!	thread callback
//!	@retval			---
//-----------------------------------------------------------------------
static
void* ThreadFuncStatic
		(
		void	*param
		)
{
	Thread	*pthis = ( Thread* )param;
	if( pthis != 0 )
	{
		pthis->ThreadProc_oride();
		pthis->m_Endflag.atomic_set( 0 );
	}
	return 0;
}
#endif

// override functions
public:
//=======================================================================
//!	thread callback
//!	@retval			---
//-----------------------------------------------------------------------
virtual
void ThreadProc_oride()
{
}

// private functions
private:
// protect functions
protected:
// public functions
public:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
Thread()
#ifdef WIN32
	: m_hThread( NULL ) , m_ThreadID( -1 )
#elif defined(__APPLE__) || defined(__linux__)
	: m_hThread( 0 ) , m_Endflag( 0 )
#endif
{
}
//=======================================================================
//!	destruct
//-----------------------------------------------------------------------
virtual
~Thread()
{
	WaitThreadEnd();
}
//=======================================================================
//!	create thread
//!	@retval			---
//-----------------------------------------------------------------------
bool CreateThread()
{
	WaitThreadEnd();

#ifdef WIN32
	m_hThread = ::CreateThread
		(
        NULL,
        0,
        ( LPTHREAD_START_ROUTINE )ThreadFuncStatic,
        ( LPVOID )this,
        0,
        &m_ThreadID);
	if( m_hThread == NULL )
		return false;
#elif defined(__APPLE__) || defined(__linux__)
	m_Endflag.normal_set( 1 );
	if( 0 != pthread_create
		( 
		&m_hThread , 
		NULL , 
		ThreadFuncStatic , 
		this
		) )
	{
		m_Endflag.normal_set( 0 );
		return false;
	}
#endif

	return true;
}
//=======================================================================
//!	check threadend
//!	@retval			---
//-----------------------------------------------------------------------
bool IsThreadEnd()
{
#ifdef WIN32
	if( m_hThread == NULL )
		return true;
	DWORD	ret = ::WaitForSingleObject( m_hThread , 0 );
	if( ret != WAIT_OBJECT_0 )
		return false;
	return true;
#elif defined(__APPLE__) || defined(__linux__)
	if( 0 == m_Endflag.atomic_get() )
		return true;
	return false;
#endif
}
//=======================================================================
//!	block until thread end
//!	@retval			---
//-----------------------------------------------------------------------
void WaitThreadEnd()
{
#ifdef WIN32
	if( m_hThread == NULL )
		return;
	::WaitForSingleObject( m_hThread , INFINITE );
	::CloseHandle( m_hThread );
	m_hThread = NULL;
#elif defined(__APPLE__) || defined(__linux__)
	if( m_hThread == 0 )
		return;
	pthread_join( m_hThread , NULL );
	m_hThread	= 0;
#endif	
}
//=======================================================================
//!	set thread priority
//!	@retval			---
//-----------------------------------------------------------------------
bool SetPriority
		(
		Priority	priority
		)
{
#ifdef WIN32
	switch( priority )
	{
	case Lowest_Priority:
		real_verify( TRUE == SetThreadPriority( m_hThread , THREAD_PRIORITY_LOWEST ) );
		return true;
	case BelowNormal_Priority:
		real_verify( TRUE == SetThreadPriority( m_hThread , THREAD_PRIORITY_BELOW_NORMAL ) );
		return true;
	case Normal_Priority:
		real_verify( TRUE == SetThreadPriority( m_hThread , THREAD_PRIORITY_NORMAL ) );
		return true;
	case AboveNormal_Priority:
		real_verify( TRUE == SetThreadPriority( m_hThread , THREAD_PRIORITY_ABOVE_NORMAL ) );
		return true;
	case Highest_Priority:
		real_verify( TRUE == SetThreadPriority( m_hThread , THREAD_PRIORITY_HIGHEST ) );
		return true;
	};
	return false;
#elif defined(__APPLE__) || defined(__linux__)
	return false;
#endif
}
//=======================================================================
//!	get thread priority
//!	@retval			---
//-----------------------------------------------------------------------
Priority GetPriority()
{
#ifdef WIN32
	int		priority = GetThreadPriority( m_hThread );
	switch( priority )
	{
	case THREAD_PRIORITY_LOWEST:
		return Lowest_Priority;
	case THREAD_PRIORITY_BELOW_NORMAL:
		return BelowNormal_Priority;
	case THREAD_PRIORITY_NORMAL:
		return Normal_Priority;
	case THREAD_PRIORITY_ABOVE_NORMAL:
		return AboveNormal_Priority;
	case THREAD_PRIORITY_HIGHEST:
		return Highest_Priority;
	default:
		return Unknown_Priority;
	};
#elif defined(__APPLE__) || defined(__linux__)
	return Unknown_Priority;
#endif	
}
//=======================================================================
//!	set processor
//!	@retval			---
//-----------------------------------------------------------------------
bool SetProcessor
		(
		int		processor_id
		);
//=======================================================================
//!	sleep
//!	@retval			---
//-----------------------------------------------------------------------
void Sleep
		(
		uint32	msec
		)
{
#ifdef WIN32
	::Sleep( msec );
#elif defined(__APPLE__) || defined(__linux__)
	timespec	treq;
	treq.tv_sec		= msec / 1000;
	treq.tv_nsec	= ( msec % 1000 ) * 1000000;
	nanosleep( &treq , NULL );
#endif
}
};

///////////////////////////////////////////////////////////////////////////
// global variable define

///////////////////////////////////////////////////////////////////////////
// global functions define

};	//namespace Thread1

//using namespace Thread1;		

#endif

