/****************************************************************************/
/*!
   	@file		DllMain1.cpp
 	
	@author 	Fanzo
 	@date 		2007/10/16
*/
/****************************************************************************/
#include			"DllMain1_pri.h" 

namespace DllMain1
{

/****************************************************************************
"DllMain" class define
****************************************************************************/
///////////////////////////////////////////////////////////////////////////
// private functions

///////////////////////////////////////////////////////////////////////////
// protected functions

///////////////////////////////////////////////////////////////////////////
// public functions

//=======================================================================
//!	construct
//-----------------------------------------------------------------------
DllMain::DllMain() : m_InitializeCount( 0 )
{
#ifdef WIN32
	__real_TlsIndex	= TlsAlloc();
	real_assert( __real_TlsIndex != 0xFFFFFFFF , "TlsAlloc failed." );
#elif __APPLE__
	real_verify( noErr == MPAllocateTaskStorageIndex( &__real_TlsIndex ) );
#elif __linux__
	real_verify( 0 == pthread_key_create( &__real_TlsIndex , NULL ) ); 
#endif
}
//=======================================================================
//!	destruct
//-----------------------------------------------------------------------
DllMain::~DllMain()
{
#ifdef WIN32
	real_verify( TRUE == TlsFree( __real_TlsIndex ) );
#elif __APPLE__
	real_verify( noErr == MPDeallocateTaskStorageIndex( __real_TlsIndex ) );
	__real_TlsIndex = 0;
#elif __linux__
	real_verify( 0 == pthread_key_delete( __real_TlsIndex ) );
	__real_TlsIndex = 0;
#endif
}
//=======================================================================
//!	Initialize
//!	@retval			---
//-----------------------------------------------------------------------
void DllMain::InitializeReal()
{
	Thread1::Autolock		autolock( m_InitializeCS );

	if( m_InitializeCount == 0 )
	{
		GC::Start();
	}
	m_InitializeCount++;
}
//=======================================================================
//!	Uninitialize
//!	@retval			---
//-----------------------------------------------------------------------
void DllMain::UninitializeReal()
{
	Thread1::Autolock		autolock( m_InitializeCS );

	m_InitializeCount--;
	real_assert( m_InitializeCount >= 0 , "Counter detected minus value." );
	if( m_InitializeCount == 0 )
	{
		GC::End();
	}
}


};//namespace DllMain1


//=======================================================================
//!	dllmain
//!	@retval			---
//-----------------------------------------------------------------------
/*
#ifdef WIN32
BOOL WINAPI DllMain
		(
		HINSTANCE	hinst ,
		DWORD		reason ,
		LPVOID		lpvReserved
		)
{
	if( reason == DLL_PROCESS_ATTACH )
	{
		__real_TlsIndex	= TlsAlloc();
		real_assert( __real_TlsIndex != 0xFFFFFFFF , "TlsAlloc failed." );
	}
	else if( reason == DLL_PROCESS_DETACH )
	{
		real_verify( TRUE == TlsFree( __real_TlsIndex ) );
	}
    return  TRUE;
}
#elif __APPLE__
#endif
*/

//=======================================================================
//!	Initialize
//!	@retval			---
//-----------------------------------------------------------------------
real_dll_export
void __real_InitializeReal()
{
	m_Main.InitializeReal();
}
//=======================================================================
//!	Uninitialize
//!	@retval			---
//-----------------------------------------------------------------------
real_dll_export
void __real_UninitializeReal()
{
	m_Main.UninitializeReal();
}
//=======================================================================
//!	GCを登録する
//!	@retval			---
//-----------------------------------------------------------------------
real_dll_export
bool __real_RegistRealGC
		(
		real::gc_info		*pinfo
		)
{
	real_assert( m_Main.m_InitializeCount != 0 , "RegistRealGC is called before initialize." );
	m_Main.RegistGC( pinfo );
	return true;
}
//=======================================================================
//!	execute collrect sync
//!	@retval			---
//-----------------------------------------------------------------------
real_dll_export
void __real_CollectGC
		(
		bool	sync , 
		bool	integrate
		)
{
	m_Main.Collect( sync , integrate );
}
//=======================================================================
//!	get memory size
//!	@retval			---
//-----------------------------------------------------------------------
real_dll_export
uint32 __real_GetMemorySize()
{
	return m_Main.GetMemorySize();
}
//=======================================================================
//!	set collect setting
//!	@retval			---
//-----------------------------------------------------------------------
real_dll_export
void __real_SetCollectSetting
		(
		CollectSetting	&setting
		)
{
	m_Main.SetCollectSetting( setting );
}
//=======================================================================
//!	get collect setting
//!	@retval			---
//-----------------------------------------------------------------------
real_dll_export
void __real_GetCollectSetting
		(
		CollectSetting	*psetting
		)
{
	m_Main.GetCollectSetting( psetting );
}
//=======================================================================
//!	set gc thread priority
//!	@retval			---
//-----------------------------------------------------------------------
real_dll_export
bool __real_SetGCThreadPriority
		(
		GCThreadPriority	priority
		)
{
	return m_Main.SetGCThreadPriority( priority );
}
//=======================================================================
//!	get gc thread priority
//!	@retval			---
//-----------------------------------------------------------------------
real_dll_export
GCThreadPriority __real_GetGCThreadPriority()
{
	return m_Main.GetGCThreadPriority();
}
//=======================================================================
//!	set gc thread processor
//!	@retval			---
//-----------------------------------------------------------------------
real_dll_export
bool __real_SetGCThreadProcessor
		(
		int		processor_id
		)
{
	return m_Main.SetGCThreadProcessor( processor_id );
}


