/****************************************************************************/
/*!
   	@file		GC1.cpp
	@author 	Fanzo
 	@date 		2007/10/25
*/
/****************************************************************************/
#include			"GC1_pri.h" 

namespace GC1
{
/****************************************************************************
"GC" class define
****************************************************************************/
///////////////////////////////////////////////////////////////////////////
// thread functions
//=======================================================================
//!	thread callback
//!	@retval			---
//-----------------------------------------------------------------------
void GC::ThreadProc_oride()
{
	while( true )
	{
		CollectSetting			setting;
		{
			Thread1::Autolock		autolock( m_SettingCS );
			setting		= m_Setting;
		}
		
		CommandQue::Command		command;
		CommandQue::Command		*plast	= 0;
		if( false == m_CommandQue.Pop( &command , &plast , setting.m_PollingTime ) )
		{
			CollectAll();
		}
		else
		{
			if( command.m_Id == Quit_CommandId )
			{
				// collect twice
				CollectAll();
				real_assert( GetMemorySize() == 0 , "GC: %d byte memory leak" , GetMemorySize() );
				return;
			}
			else if( command.m_Id == Collect_CommandId )
			{
				CollectAll();
			}
			else if( command.m_Id == CollrectSetting_CommandId )
			{
			}
		}
		// call event
		CommandQue::Command		*pfirst = &command;
		while( pfirst != 0 && pfirst->m_Id != Null_CommandId )
		{
			if( pfirst->m_pEvent != 0 )
			{
				pfirst->m_pEvent->SetEvent();
				pfirst->m_pEvent = 0;
			}
			pfirst->m_Id = Null_CommandId;
			CommandQue::Command		*t = pfirst->m_Next;
			pfirst->m_Next = 0;
			pfirst = t;
		}
	}
}
//=======================================================================
//!	get collect list
//!	@retval			---
//-----------------------------------------------------------------------
void GC::GetCollectList
		(
		gc_info	**start , 
		gc_info	**end
		)
{
	// add terminate
	gc_info	*term = CreateTerminate();
	RegistTerm( term );

	*start	= m_Start;
	*end	= term;
}
//=======================================================================
//!	regist gc_info
//!	@retval			---
//-----------------------------------------------------------------------
void GC::RegistTerm
		(
		gc_info	*info
		)
{
	// regist gc
	m_CS.Lock();
	m_Last->m_Next	= info;
	m_Last			= info;
	m_CS.Unlock();
}
//=======================================================================
//!	clear
//!	@retval			---
//-----------------------------------------------------------------------
void GC::ClearList
		(
		gc_info	*start , 
		gc_info	*end
		)
{
	gc_info	*now	= start;
	while( now != end )
	{
		now->m_State.atomic_and( 0x7FFFFFFF );
		now = now->m_Next;
	}
}
//=======================================================================
//!	clear trace
//!	@retval			---
//-----------------------------------------------------------------------
void GC::ClearTrace
		(
		gc_info	*start , 
		gc_info	*end
		)
{
	gc_info	*now	= start;
	while( now != end )
	{
		now->m_Trace = false;
		now = now->m_Next;
	}
}
//=======================================================================
//!	trace tree
//!	@retval			---
//-----------------------------------------------------------------------
void GC::TraceTree
		(
		gc_info	*now
		)
{
	if( now == 0 )
		return;
	if( now->m_AllocType == Null_AllocType )
	{
	}
	else if( now->m_AllocType == Object_AllocType )
	{
		if( now->m_Trace == false )
		{
			now->m_Trace	= true;
			gc_object_info	*info = ( gc_object_info* )now;
			gc_ptr			*next = info->m_NextPtr.atomic_get();
			while( next != 0 )
			{
				TraceTree( next->get_atomic_ptr() );
				next = next->m_Next.atomic_get();
			}
		}
	}
	else if( now->m_AllocType == Array_AllocType )
	{
		if( now->m_Trace == false )
		{
			now->m_Trace	= true;
			gc_array_info	*info = ( gc_array_info* )now;

			uint32	off;
			uint8	*p	= ( uint8* )info->m_pArray;
			for( off = 0 ; off < info->m_ElementNum ; off++ )
			{
				atomic_ptr< gc_info >	*ptr = ( atomic_ptr< gc_info >* )( p + info->m_ElementPointerOffset );
				TraceTree( ptr->atomic_get() );
				p += info->m_ElementSize;
			}
		}
	}
}
//=======================================================================
//!	trace
//!	@retval			---
//-----------------------------------------------------------------------
bool GC::Trace
		(
		gc_info	*start , 
		gc_info	*end
		)
{
	bool			exist = false;
	gc_info	*now	= start;
	while( now != end )
	{
		if( now->m_AllocType == Null_AllocType )
		{
		}
		else if( now->m_AllocType == Object_AllocType )
		{
			if( now->m_Trace == false && now->m_State.atomic_get() > 0 )
			{
				exist			= true;
				now->m_Trace	= true;
				gc_object_info	*info = ( gc_object_info* )now;
				gc_ptr			*next = info->m_NextPtr.atomic_get();
				while( next != 0 )
				{
					TraceTree( next->get_atomic_ptr() );
					next = next->m_Next.atomic_get();
				}
			}
		}
		else if( now->m_AllocType == Array_AllocType )
		{
			if( now->m_Trace == false && now->m_State.atomic_get() > 0 )
			{
				exist			= true;
				now->m_Trace	= true;
				gc_array_info	*info = ( gc_array_info* )now;

				uint32	off;
				uint8	*p	= ( uint8* )info->m_pArray;
				for( off = 0 ; off < info->m_ElementNum ; off++ )
				{
					atomic_ptr< gc_info >	*ptr = ( atomic_ptr< gc_info >* )( p + info->m_ElementPointerOffset );
					TraceTree( ptr->atomic_get() );
					p += info->m_ElementSize;
				}
			}
		}
		// update
		now = now->m_Next;
	}
	return exist;
}
//=======================================================================
//!	remove trace
//!	@retval			---
//-----------------------------------------------------------------------
bool GC::RemoveTrace
		(
		gc_info	*start , 
		gc_info	*end , 
		gc_info	**t_start , 
		gc_info	**t_end
		)
{
	bool	exist = false;
	gc_info	*prev	= start;
	gc_info	*now	= start->m_Next;

	while( now != end )
	{
		if( now->m_Trace == true )
		{
			exist = true;
			prev->m_Next = now->m_Next;
			now->m_Next = 0;
			if( *t_start == 0 )
			{
				*t_start = now;
				*t_end	 = now;
			}
			else
			{
				( *t_end )->m_Next = now;
				*t_end	= now;
			}
			now = prev->m_Next;
		}
		else
		{
			prev = now;
			now	 = now->m_Next;//prev->m_Next;
		}
	}
	return exist;
}
//=======================================================================
//!	remove
//!	@retval			---
//-----------------------------------------------------------------------
void GC::Delete
		(
		gc_info	*start , 
		gc_info	*end
		)
{
	gc_info	*now	= start->m_Next;
	while( now != end )
	{
		gc_info	*next = now->m_Next;
		m_MemorySize.atomic_sub( now->m_Size );
		if( now->m_DeleteGC != 0 )
		{
			now->m_DeleteGC(  now , now->m_DeleteGCParam );
		}
		now	= next;
	}
	start->m_Next = end;
}
//=======================================================================
//!	Collect GC
//!	@retval			---
//-----------------------------------------------------------------------
void GC::CollectAll()
{
	real_trace( "CollectAll\n" );
	
	gc_info	*start , *end;
	
	GetCollectList( &start , &end );
	ClearList( start , end );
	GetCollectList( &start , &end );
	
	ClearTrace( start , end );
	gc_info	*t_start = 0 , *t_end = 0;
	while( true )
	{
		Trace( start , end );
		if( false == RemoveTrace( start , end , &t_start , &t_end ) )
			break;
	}
	Delete( start , end );

	if( t_start != 0 )
	{
		start->m_Next = t_start;
		t_end->m_Next = end;
	}
}
///////////////////////////////////////////////////////////////////////////
// private functions

//=======================================================================
//!	get terminate
//!	@retval			---
//-----------------------------------------------------------------------
gc_info* GC::CreateTerminate()
{
	gc_info	*p = &m_Terminates[ m_TerminateCount ];
	m_TerminateCount++;
	if( m_TerminateCount >= TERMINATE_NUM )
		m_TerminateCount  = 0;
	p->m_Size = 0;
	p->m_Next = 0;
	return p;
}
///////////////////////////////////////////////////////////////////////////
// protected functions

///////////////////////////////////////////////////////////////////////////
// public functions

//=======================================================================
//!	construct
//-----------------------------------------------------------------------
GC::GC() :  
		m_CommandQue( 512 ) , 
		m_TerminateCount( 0 ) , 
		m_Start( 0 ) , 
		m_Last( 0 )
{
	// initialize setting
//	m_Setting.m_PollingTime	= 300;
	
	// initialize terminate
	int		off;
	for( off = 0 ; off < TERMINATE_NUM ; off++ )
	{
		m_Terminates[ off ].m_AllocType	= Null_AllocType;
		m_Terminates[ off ].m_DeleteGC	= 0;
		m_Terminates[ off ].m_Trace		= false;
		m_Terminates[ off ].m_Next		= 0;
		m_Terminates[ off ].m_Size		= 0;
		m_Terminates[ off ].m_DeleteGCParam	= 0;
	}
	m_StartInfo.m_AllocType	= Null_AllocType;
	m_StartInfo.m_DeleteGC	= 0;
	m_StartInfo.m_Trace		= false;
	m_StartInfo.m_Next		= 0;
	m_StartInfo.m_Size		= 0;
	m_StartInfo.m_DeleteGCParam	= 0;
	
	m_Start	= &m_StartInfo;
	m_Last	= m_Start;
}
//=======================================================================
//!	start
//!	@retval			---
//-----------------------------------------------------------------------
bool GC::Start()
{
	End();
	if( false == CreateThread() )
		return false;
//	SetPriority( Thread1::AboveNormal_Priority );
	return true;
}
//=======================================================================
//!	end
//!	@retval			---
//-----------------------------------------------------------------------
void GC::End()
{
	CommandQue::Command		command;
	command.m_Id	= Quit_CommandId;
	m_CommandQue.Push( command );
	WaitThreadEnd();
	m_CommandQue.Reset();
}
//=======================================================================
//!	regist gc_info
//!	@retval			---
//-----------------------------------------------------------------------
void GC::RegistGC
		(
		gc_info	*info
		)
{
	// memory check
	CollectSetting	setting;
	{
		Thread1::Autolock	lock( m_SettingCS );
		setting = m_Setting;
	}
	uint32		size = m_MemorySize.atomic_get();
	if( setting.m_SyncCollectMemorySize != 0 && setting.m_SyncCollectMemorySize <= size )
	{
		Collect( true , true );
	}
	else if( setting.m_AsyncCollectMemorySize != 0 && setting.m_AsyncCollectMemorySize <= size )
	{
		Collect( false , true );
	}

	// update memorysize
	m_MemorySize.atomic_add( info->m_Size );
	
	// regist gc
	m_CS.Lock();
	m_Last->m_Next	= info;
	m_Last			= info;
	m_CS.Unlock();
}
//=======================================================================
//!	execute collrect sync
//!	@retval			---
//-----------------------------------------------------------------------
void GC::Collect
		(
		bool	sync , 
		bool	integrate
		)
{
	if( sync == true )
	{
		Thread1::Event		s_event;
		
		CommandQue::Command	command;
		command.m_Id	= Collect_CommandId;
		command.m_pEvent= &s_event;
		command.m_Async = integrate == true ? true : false;
		m_CommandQue.Push( command );
		
		// wait finish
		s_event.Wait();	
	}
	else
	{
		CommandQue::Command	command;
		command.m_Id	= Collect_CommandId;
		command.m_Async = integrate == true ? true : false;
		m_CommandQue.Push( command );
	}
/*
	if( sync == true )
	{
		SharedHandle1::shared_handle< Thread1::Event >	sync( new Thread1::Event() );
		
		CommandData	command;
		command.m_Command		= Collect_Command;
		command.m_FinishEvent	= sync;
		m_CommandQue.Push( command );
		
		// wait finish
		sync->Wait();	
	}
	else
	{
		CommandData	command;
		command.m_Command	= Collect_Command;
		m_CommandQue.Push( command );
	}
*/
}
//=======================================================================
//!	get memory size
//!	@retval			---
//-----------------------------------------------------------------------
uint32 GC::GetMemorySize()
{
	return m_MemorySize.atomic_get();
}
//=======================================================================
//!	set collect setting
//!	@retval			---
//-----------------------------------------------------------------------
void GC::SetCollectSetting
		(
		CollectSetting	&setting
		)
{
	Thread1::Autolock		autolock( m_SettingCS );
	m_Setting	= setting;
}
//=======================================================================
//!	get collect setting
//!	@retval			---
//-----------------------------------------------------------------------
void GC::GetCollectSetting
		(
		CollectSetting	*psetting
		)
{
	Thread1::Autolock		autolock( m_SettingCS );
	*psetting = m_Setting;
}
//=======================================================================
//!	set gc thread priority
//!	@retval			---
//-----------------------------------------------------------------------
bool GC::SetGCThreadPriority
		(
		GCThreadPriority	priority
		)
{
	switch( priority )
	{
	case Lowest_GCThreadPriority:
		return SetPriority( Thread1::Lowest_Priority );
	case BelowNormal_GCThreadPriority:
		return SetPriority( Thread1::BelowNormal_Priority );
	case Normal_GCThreadPriority:
		return SetPriority( Thread1::Normal_Priority );
	case AboveNormal_GCThreadPriority:
		return SetPriority( Thread1::AboveNormal_Priority );
	case Highest_GCThreadPriority:
		return SetPriority( Thread1::Highest_Priority );
	default:
		return false;
	}
}
//=======================================================================
//!	get gc thread priority
//!	@retval			---
//-----------------------------------------------------------------------
GCThreadPriority GC::GetGCThreadPriority()
{
	switch( GetPriority() )
	{
	case Thread1::Lowest_Priority:
		return Lowest_GCThreadPriority;
	case Thread1::BelowNormal_Priority:
		return BelowNormal_GCThreadPriority;
	case Thread1::Normal_Priority:
		return Normal_GCThreadPriority;
	case Thread1::AboveNormal_Priority:
		return AboveNormal_GCThreadPriority;
	case Thread1::Highest_Priority:
		return Highest_GCThreadPriority;
	default:
		return Other_GCThreadPriority;
	}
}
//=======================================================================
//!	set gc thread processor
//!	@retval			---
//-----------------------------------------------------------------------
bool GC::SetGCThreadProcessor
		(
		int		processor_id
		)
{
	return SetProcessor( processor_id );
}

};//namespace GC1

