/****************************************************************************/
/*!
   	@file		GC1_pub.h
	@author 	Fanzo
 	@date 		2007/10/25
*/
/****************************************************************************/

#ifndef		GC1_pub_h
#define		GC1_pub_h

///////////////////////////////////////////////////////////////////////////
//include files
#include	"Real.h"
#include	"../../../../SharedSource/common/Thread1/CriticalSection1/CriticalSection1_pub.h"
#include	"../../../../SharedSource/common/Thread1/Thread1/Thread1_pub.h"
#include	"../../../../SharedSource/common/Thread1/Que1/Que1_pub.h"
#include	"../../../../SharedSource/common/Thread1/Event1/Event1_pub.h"
#include	"../../../../SharedSource/common/SharedHandle1/SharedHandle1_pub.h"
#include	"../../../../SharedSource/common/Thread1/Semaphore1/Semaphore1_pub.h"


namespace GC1
{
using namespace real;

///////////////////////////////////////////////////////////////////////////
// preprocessor deifne
#define		TERMINATE_NUM		3

///////////////////////////////////////////////////////////////////////////
// type define
enum CommandId
{
	Null_CommandId	= 0 , 
	Quit_CommandId , 
	Collect_CommandId , 
	CollrectSetting_CommandId , 
	
	CommandNum_CommandId , 
};

///////////////////////////////////////////////////////////////////////////
// classes define

/****************************************************************************
"CommandQue" class 
****************************************************************************/
/*===========================================================================
//!	
===========================================================================*/
/////////////////////////////////////////////////////////////////////////////
class CommandQue
{
	real_copy_impossible( CommandQue );
	
public:	
	class Command
	{
	public:
		CommandId		m_Id;
		bool			m_Async;
		Command			*m_Next;
		Thread1::Event	*m_pEvent;
		
		Command() : 
			m_Id( Null_CommandId ) , 
			m_Async( false ) , 
			m_Next( 0 ) , 
			m_pEvent( 0 )	
		{
		}
	};	
// variable member
private:
	Array1::Array< Command >	m_Que;
	int							m_ReadPos;
	int							m_WritePos;
	Thread1::CriticalSection	m_CS;
	Thread1::Semaphore			m_Count;
	Thread1::Semaphore			m_Empty;

	Command*					m_LastTable[ CommandNum_CommandId ];
	
// private functions
private:
// protect functions
protected:
// public functions
public:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
CommandQue
	(
	int32	quemax
	) 
	:m_Que( quemax ) , 
	m_ReadPos( 0 ) , 
	m_WritePos( 0 ) , 
	m_Count( 0 , quemax ) , 
	m_Empty( quemax , quemax )
{
	int		off;
	for( off = 0 ; off < CommandNum_CommandId ; off++ )
		m_LastTable[ off ] = 0;
}
//=======================================================================
//!	destruct
//-----------------------------------------------------------------------
~CommandQue()
{
}
//=======================================================================
//!	reset
//!	@retval			---
//-----------------------------------------------------------------------
void Reset()
{
	Command	data;
	Command	*last;
	while( true == Pop( &data , &last , 0 ) )
	{
		if( data.m_pEvent != 0 )
			data.m_pEvent->SetEvent();
	}
}
//=======================================================================
//!	push
//!	@retval			---
//-----------------------------------------------------------------------
void Push
		(
		Command	&data
		)
{
	if( data.m_Async == false )
	{
		m_Empty.Wait();
		m_CS.Lock();
		m_Que[ m_WritePos ] = data;
		m_WritePos++;
		if( m_WritePos == m_Que.GetDataNum() )
			m_WritePos = 0;
		m_CS.Unlock();
		m_Count.Post();
	}
	else
	{
		m_Empty.Wait();
		m_CS.Lock();
		m_Que[ m_WritePos ] = data;
		
		// set link
		if( m_LastTable[ data.m_Id ] == 0 )
		{
			m_LastTable[ data.m_Id ] = &m_Que[ m_WritePos ];
		}
		else
		{
			m_LastTable[ data.m_Id ]->m_Next = &m_Que[ m_WritePos ];
			m_LastTable[ data.m_Id ] = &m_Que[ m_WritePos ];
		}	

		m_WritePos++;
		if( m_WritePos == m_Que.GetDataNum() )
			m_WritePos = 0;
		m_CS.Unlock();
		m_Count.Post();
	}
}
//=======================================================================
//!	pop timeout
//!	@retval			if timeout , return false
//-----------------------------------------------------------------------
bool Pop
		(
		Command		*pdata , 
		Command		**plast ,			
		uint32		msec				//!< [in] if 0xFFFFFFF , wait infinity.
		)
{
	if( msec == 0xFFFFFFFF )
		m_Count.Wait();
	else if( false == m_Count.WaitTimeout( msec ) )
		return false;

	m_CS.Lock();
	*pdata = m_Que[ m_ReadPos ];
	
	if( pdata->m_Async == true )
	{
		if( m_LastTable[ pdata->m_Id ] == &m_Que[ m_ReadPos ] )
			*plast = pdata;
		else
			*plast = m_LastTable[ pdata->m_Id ];
		m_LastTable[ pdata->m_Id ]	= 0;
	}
	else
		*plast = pdata;
	
	m_ReadPos++;
	if( m_ReadPos == m_Que.GetDataNum() )
		m_ReadPos = 0;
	m_CS.Unlock();
	m_Empty.Post();
	return true;
}
};

/****************************************************************************
"GC" class 
****************************************************************************/
/*===========================================================================
//!	
===========================================================================*/
/////////////////////////////////////////////////////////////////////////////
class GC : public Thread1::Thread
{
private:

// variable member
private:
	// control
	Thread1::CriticalSection			m_SettingCS;
	CollectSetting						m_Setting;
	CommandQue							m_CommandQue;
	
	// GC
	int									m_TerminateCount;
	gc_info								m_StartInfo;
	gc_info								m_Terminates[ TERMINATE_NUM ];

	Thread1::CriticalSection			m_CS;
	gc_info								*m_Start;
	gc_info								*m_Last;

	// GC info
	atomic_uint32						m_MemorySize;

// thread functions
private:
//=======================================================================
//!	thread callback
//!	@retval			---
//-----------------------------------------------------------------------
void ThreadProc_oride();
//=======================================================================
//!	get collect list
//!	@retval			---
//-----------------------------------------------------------------------
void GetCollectList
		(
		gc_info	**start , 
		gc_info	**end
		);
//=======================================================================
//!	regist gc_info
//!	@retval			---
//-----------------------------------------------------------------------
void RegistTerm
		(
		gc_info	*info
		);
//=======================================================================
//!	clear
//!	@retval			---
//-----------------------------------------------------------------------
void ClearList
		(
		gc_info	*start , 
		gc_info	*end
		);
//=======================================================================
//!	clear trace
//!	@retval			---
//-----------------------------------------------------------------------
void ClearTrace
		(
		gc_info	*start , 
		gc_info	*end
		);
//=======================================================================
//!	trace tree
//!	@retval			---
//-----------------------------------------------------------------------
void TraceTree
		(
		gc_info	*now
		);
//=======================================================================
//!	trace
//!	@retval			---
//-----------------------------------------------------------------------
bool Trace
		(
		gc_info	*start , 
		gc_info	*end
		);
//=======================================================================
//!	remove trace
//!	@retval			---
//-----------------------------------------------------------------------
bool RemoveTrace
		(
		gc_info	*start , 
		gc_info	*end , 
		gc_info	**t_start , 
		gc_info	**t_end
		);
//=======================================================================
//!	remove
//!	@retval			---
//-----------------------------------------------------------------------
void Delete
		(
		gc_info	*start , 
		gc_info	*end
		);
//=======================================================================
//!	Collect GC
//!	@retval			---
//-----------------------------------------------------------------------
void CollectAll();

// private functions
private:
//=======================================================================
//!	get terminate
//!	@retval			---
//-----------------------------------------------------------------------
gc_info* CreateTerminate();
// protect functions
protected:
// public functions
public:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
GC();
//=======================================================================
//!	start
//!	@retval			---
//-----------------------------------------------------------------------
bool Start();
//=======================================================================
//!	end
//!	@retval			---
//-----------------------------------------------------------------------
void End();
//=======================================================================
//!	regist gc_info
//!	@retval			---
//-----------------------------------------------------------------------
void RegistGC
		(
		gc_info	*info
		);
//=======================================================================
//!	execute collrect sync
//!	@retval			---
//-----------------------------------------------------------------------
void Collect
		(
		bool	sync , 
		bool	integrate
		);
//=======================================================================
//!	get memory size
//!	@retval			---
//-----------------------------------------------------------------------
uint32 GetMemorySize();
//=======================================================================
//!	set collect setting
//!	@retval			---
//-----------------------------------------------------------------------
void SetCollectSetting
		(
		CollectSetting	&setting
		);
//=======================================================================
//!	get collect setting
//!	@retval			---
//-----------------------------------------------------------------------
void GetCollectSetting
		(
		CollectSetting	*psetting
		);
//=======================================================================
//!	set gc thread priority
//!	@retval			---
//-----------------------------------------------------------------------
bool SetGCThreadPriority
		(
		GCThreadPriority	priority
		);
//=======================================================================
//!	get gc thread priority
//!	@retval			---
//-----------------------------------------------------------------------
GCThreadPriority GetGCThreadPriority();
//=======================================================================
//!	set gc thread processor
//!	@retval			---
//-----------------------------------------------------------------------
bool SetGCThreadProcessor
		(
		int		processor_id
		);
};

///////////////////////////////////////////////////////////////////////////
// global variable define

///////////////////////////////////////////////////////////////////////////
// global functions define

};	//namespace GC1

//using namespace GC1;		

#endif

