/****************************************************************************/
/*!
   	@file		Debug.h
	@author 	Fanzo
 	@date 		2007/10/18
*/
/****************************************************************************/

#ifndef		real_Debug_h
#define		real_Debug_h

///////////////////////////////////////////////////////////////////////////
//include files
#ifdef WIN32
	#include	<windows.h>
	#include	<stdio.h>
#elif __linux__
	#include	<stdio.h>
	#include 	<stdarg.h>
	#include	<assert.h>
#endif


#pragma pack( push , 1 )		//set align

namespace real
{

///////////////////////////////////////////////////////////////////////////
// preprocessor deifne

#ifdef _DEBUG
	#define		real_verify( exp )										\
	{																	\
		real_assert( exp , "[verify assert]\n" #exp "\n" );				\
	}																
#else
	#define		real_verify( exp )	((void)(exp))
#endif

#if defined(__APPLE__) || defined(__linux__)

#ifdef _DEBUG
	#define		real_assert( ck , ... )								\
	{																\
		if( ck == false )											\
		{															\
			real::Assert( __FILE__ , __LINE__ , __VA_ARGS__ );		\
		}															\
	}
	#define		real_trace( ... )									\
	{																\
		real::Trace( __VA_ARGS__ );									\
	}			
#else
	#define		real_assert( ck , ... )	
	#define		real_trace( ... )	
#endif


#elif WIN32

#ifdef _DEBUG
	#define		real_assert		real::AssertInfo( __FILE__ , __LINE__ )	

	#define		real_trace		real::TraceInfo()

#else
	#define		real_assert( ck , ... )	
	#define		real_trace( ... )	
#endif

#else
	#error		Debug1:Invalid OS
#endif

///////////////////////////////////////////////////////////////////////////
// type define

///////////////////////////////////////////////////////////////////////////
// classes define

#ifdef _DEBUG
//=======================================================================
//!	disp message box.
//!	@retval			---
//-----------------------------------------------------------------------
inline
bool AssertDialog
		(
		const char		*pmes
		)
{
#ifdef __APPLE__
	DialogRef	alert;
	CFStringRef	mes = CFStringCreateWithCString( kCFAllocatorDefault , pmes , kCFStringEncodingShiftJIS );

	AlertStdCFStringAlertParamRec	params;
	GetStandardAlertDefaultParams( &params , kStdCFStringAlertVersionOne );
    params.defaultText		= CFSTR( "Continue" );
    params.defaultButton	= kAlertStdAlertOKButton;  
    params.cancelText		= CFSTR( "Stop" );
    params.cancelButton	= kAlertStdAlertCancelButton;
	CreateStandardAlert( kAlertStopAlert,  CFSTR( "assert" ) , mes , &params , &alert );
	DialogItemIndex	hit;
    RunStandardAlert( alert , NULL, &hit );
	CFRelease( mes );

	if( hit == 1 )
		return false;
	return true;

#elif WIN32
	if( IDCANCEL == ::MessageBoxA( NULL , pmes , "ASSERT" , MB_RETRYCANCEL | MB_SYSTEMMODAL | MB_ICONERROR ) )
		return true;
	return false;

#elif __linux__
	return false;
#else
	#error	Unknown OS.
#endif
}
//=======================================================================
//!	disp
//!	@retval			---
//-----------------------------------------------------------------------
inline
void TraceExe
		(
		const char	*pmes , 
		bool		error 
		)
{
#if defined(__APPLE__) || defined(__linux__)
	if( error == true )
		fprintf( stderr , pmes );
	else
		printf( pmes );
#elif WIN32
	OutputDebugStringA( pmes );
#else
	#error basic::Debug1 Unknown OS.
#endif

}

#ifdef __APPLE__

//=======================================================================
//!	disp
//!	@retval			---
//-----------------------------------------------------------------------
inline
void Assert
		(
		const char	*pfile , 
		int			line , 
		const char	*pformat , 
		...
		)
{
	//crate string.
	char	mes[ 256 ];
	{
		va_list	args;
		va_start( args , pformat );
		vsnprintf( mes , sizeof( mes ) , pformat , args );
		va_end( args );
	}
	
	//insert filename.
	char	buf[ 255 ];
	snprintf( buf , sizeof( buf ) , "%s\nline( %d )\n%s" , pfile , line , mes );
	TraceExe( buf , true );
	if( true == AssertDialog( buf ) )
	{
		exit( 0 );
	}
}
//=======================================================================
//!	disp
//!	@retval			---
//-----------------------------------------------------------------------
inline
void Trace
		(
		const char	*pformat , 
		...
		)
{
	//create string.
	char	mes[ 256 ];
	{
		va_list	args;
		va_start( args , pformat );
		vsnprintf( mes , sizeof( mes ) , pformat , args );
		va_end( args );
	}
	TraceExe( mes , false );
}

#elif __linux__

//=======================================================================
//!	disp
//!	@retval			---
//-----------------------------------------------------------------------
inline
void Assert
		(
		const char	*pfile , 
		int			line , 
		const char	*pformat , 
		...
		)
{
	//crate string.
	char	mes[ 256 ];
	{
		va_list	args;
		va_start( args , pformat );
		vsnprintf( mes , sizeof( mes ) , pformat , args );
		va_end( args );
	}
	
	//insert filename.
	char	buf[ 255 ];
	snprintf( buf , sizeof( buf ) , "%s\nline( %d )\n%s" , pfile , line , mes );
	TraceExe( buf , true );
	
	assert( false );
}
//=======================================================================
//!	disp
//!	@retval			---
//-----------------------------------------------------------------------
inline
void Trace
		(
		const char	*pformat , 
		...
		)
{
	//create string.
	char	mes[ 256 ];
	{
		va_list	args;
		va_start( args , pformat );
		vsnprintf( mes , sizeof( mes ) , pformat , args );
		va_end( args );
	}
	TraceExe( mes , false );
}

#elif WIN32

/****************************************************************************
 AssertInfo
****************************************************************************/
/*===========================================================================
[ class purpose ]
===========================================================================*/
/////////////////////////////////////////////////////////////////////////////
class AssertInfo
{
// members 
private:
	const char		*m_pFile;
	int32			m_Line;
	
// public functions 
public:
/*===========================================================================
Construct
                                               
@retval			---
---------------------------------------------------------------------------*/
AssertInfo
		(
		const char	*pfile , 
		int32		line
		) : m_pFile( pfile ) , m_Line( line )
{
}
/*===========================================================================
operator
                                               
@retval			---
---------------------------------------------------------------------------*/
void operator()
		(
		bool		check , 
		const char	*pformat , 
		...
		)const
{
	if( check != false )
        return;

	//create string.
	char	*pmes	= 0;
	int		len		= 0;
	{
		va_list	args;
		va_start( args , pformat );
		len		= _vscprintf( pformat, args ) + 1;
		pmes	= ( char* )malloc( len * sizeof( char ) );
		vsprintf_s( pmes , len , pformat , args );
		va_end( args );
	}
	
	//insert filename.
	char	*pbuf	= ( char* )malloc( ( len + MAX_PATH + 50 ) * sizeof( char ) );
	_snprintf_s( pbuf , ( len + MAX_PATH + 50 ) , ( len + MAX_PATH + 50 ) * sizeof( char ) - 1 , "%s\nline( %d )\n%s" , m_pFile , m_Line , pmes );
	TraceExe( pbuf , true );
	if( true == AssertDialog( pbuf ) )
	{
		free( pmes );
		free( pbuf );
		exit( 0 );
	}
	free( pmes );
	free( pbuf );
}
};
/****************************************************************************
 TraceInfo
****************************************************************************/
/*===========================================================================
[ class purpose ]
===========================================================================*/
/////////////////////////////////////////////////////////////////////////////
class TraceInfo
{
// members 
private:

// public functions 
public:
/*===========================================================================
operator
                                               
@retval			---
---------------------------------------------------------------------------*/
void operator()
		(
		const char	*pformat , 
		...
		)const
{
	//create string.
	char	*pmes	= 0;
	int		len		= 0;
	{
		va_list	args;
		va_start( args , pformat );
		len		= _vscprintf( pformat, args ) + 1;
		pmes	= ( char* )malloc( len * sizeof( char ) );
		vsprintf_s( pmes , len  , pformat , args );
		va_end( args );
	}
	TraceExe( pmes , false );
	free( pmes );
}
};
#else
	#error		Debug.h:Unknown OS.
#endif



#endif		//_DEBUG

///////////////////////////////////////////////////////////////////////////
// global variable define

///////////////////////////////////////////////////////////////////////////
// global functions define

};	//namespace

//using namespace real;		

#pragma pack( pop )			//release align

#endif

