/****************************************************************************/
/*!
   	@file		GC.h
	@author 	Fanzo
 	@date 		2007/10/18
*/
/****************************************************************************/

#ifndef		real_GC_h
#define		real_GC_h

///////////////////////////////////////////////////////////////////////////
//include files


#pragma pack( push , 1 )		//set align

namespace real
{

///////////////////////////////////////////////////////////////////////////
// preprocessor deifne

///////////////////////////////////////////////////////////////////////////
// type define
struct gc_info;
typedef  void ( real_call *TypeOfDeleteGC)(gc_info*,void*);

enum AllocType
{
	Null_AllocType	= 0 , 
	Object_AllocType , 
	Array_AllocType , 
};

class gc_element_ptr;
class gc_ptr;

///////////////////////////////////////////////////////////////////////////
// classes define

/****************************************************************************
"gc_info" class 
****************************************************************************/
struct gc_info
{
	bool			m_Trace;
	gc_info*		m_Next;
	uint32			m_Size;
	TypeOfDeleteGC	m_DeleteGC;
	void*			m_DeleteGCParam;
	AllocType		m_AllocType;
	atomic_uint32	m_State;
};
/****************************************************************************
"gc_object" class 
****************************************************************************/
class gc_ptr;
struct gc_object_info
{
	gc_info				m_GCInfo;
	atomic_ptr<gc_ptr>	m_NextPtr;
};
/****************************************************************************
"gc_array" class 
****************************************************************************/
struct gc_array_info
{
	gc_info			m_GCInfo;
	uint32			m_ElementNum;
	uint32			m_ElementSize;
	uint32			m_ElementPointerOffset;
	void*			m_pArray;
};
/****************************************************************************
"gc_ptr" class 
****************************************************************************/
class gc_ptr : protected atomic_ptr<gc_info>
{
	friend class gc_element_ptr;
public:
	atomic_ptr<gc_ptr>	m_Next;
	
private:
	bool	m_Root;
public:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
gc_ptr() : m_Root( true )
{
}
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
gc_ptr
		(
		gc_info	*ptr
		) : atomic_ptr<gc_info>( ptr ) , m_Root( true )
{
}
//=======================================================================
//!	destruct
//-----------------------------------------------------------------------
virtual
~gc_ptr()
{
	if( m_Root == true )
	{
		gc_info	*t = normal_get();
		if( t != 0 )
			t->m_State.atomic_decrement();
	}	
}
//=======================================================================
//!	release
//!	@retval			---
//-----------------------------------------------------------------------
inline
void release()
{
	if( m_Root == true )
	{
		gc_info	*t = normal_get();
		if( t != 0 )
			t->m_State.atomic_decrement();
		atomic_set( 0 );
	}
	else
	{
		atomic_set( 0 );
	}
}
//=======================================================================
//!	set mode
//!	@retval			---
//-----------------------------------------------------------------------
inline
void set_mode
		(
		bool	root
		)
{
	m_Root	= root;
}
//=======================================================================
//!	substitution
//!	@retval			---
//-----------------------------------------------------------------------
inline
gc_ptr& operator=
		(
		const gc_ptr	&s
		)
{
	*this = s.get_ptr();
	return *this;
}
//=======================================================================
//!	substitution
//!	@retval			---
//-----------------------------------------------------------------------
inline
gc_ptr& operator=
		(
		gc_info	*s
		)
{
	if( m_Root == true )
	{
		if( s != 0 )
			s->m_State.atomic_increment();
		gc_info	*t = normal_get();
		if( t != 0 )
			t->m_State.atomic_decrement();
		normal_set( s );	
	}
	else
	{
		atomic_set( s );
		if( s != 0 )
			s->m_State.atomic_or( 0x80000000 );
	}
	return *this;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		bool	b
		)const
{
	return ( b == ( ( normal_get() != 0 ) ? true : false ) );
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		bool	b
		)const
{
	return ( ( *this ) == b ) ? false : true;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		const gc_ptr	&obj
		)const
{
	return ( normal_get() == obj.normal_get() ) ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		const gc_ptr	&obj
		)const
{
	return ( normal_get() != obj.normal_get() ) ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		const gc_element_ptr	&obj
		)const;
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		const gc_element_ptr	&obj
		)const;
//=======================================================================
//!	get ptr
//!	@retval			---
//-----------------------------------------------------------------------
inline
gc_info* get_ptr()const
{
	return normal_get();
}
//=======================================================================
//!	get ptr
//!	@retval			---
//-----------------------------------------------------------------------
inline
gc_info* get_atomic_ptr()const
{
	return atomic_get();
}
};

/****************************************************************************
"gc_element_ptr" class 
****************************************************************************/
class gc_element_ptr : protected atomic_ptr<gc_info>
{
	real_copy_impossible( gc_element_ptr );
	friend class gc_ptr;
public:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
gc_element_ptr()
{
}
//=======================================================================
//!	release
//!	@retval			---
//-----------------------------------------------------------------------
inline
void release()
{
	atomic_set( 0 );
}
//=======================================================================
//!	substitution
//!	@retval			---
//-----------------------------------------------------------------------
inline
gc_element_ptr& operator=
		(
		gc_info	*s
		)
{
	atomic_set( s );
	if( s != 0 )
		s->m_State.atomic_or( 0x80000000 );
	return *this;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		bool	b
		)const
{
	return ( b == ( ( normal_get() != 0 ) ? true : false ) );
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		bool	b
		)const
{
	return ( ( *this ) == b ) ? false : true;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		const gc_element_ptr	&obj
		)const
{
	return ( normal_get() == obj.normal_get() ) ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		const gc_element_ptr	&obj
		)const
{
	return ( normal_get() != obj.normal_get() ) ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		(
		const gc_ptr	&obj
		)const;
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		(
		const gc_ptr	&obj
		)const;
//=======================================================================
//!	get ptr
//!	@retval			---
//-----------------------------------------------------------------------
inline
gc_info* get_ptr()const
{
	return normal_get();
}
};

/****************************************************************************
"gc_temporaly" class 
****************************************************************************/
/////////////////////////////////////////////////////////////////////////////
template<class T>
class gc_temporaly
{
// variable member
private:
	gc_info		*m_Ptr;
	T			*m_Query;
	
// public functions
public:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
gc_temporaly
		(
		gc_info	*ptr , 
		T		*query
		) : m_Ptr( ptr ) , m_Query( query )
{
	m_Ptr->m_State.atomic_increment();
}
//=======================================================================
//!	destruct
//-----------------------------------------------------------------------
~gc_temporaly()
{
	m_Ptr->m_State.atomic_decrement();
}
//=======================================================================
//!	ref operator
//!	@retval			---
//-----------------------------------------------------------------------
inline
T* operator->()const
{
	return m_Query;
}
};

///////////////////////////////////////////////////////////////////////////
// global variable define

///////////////////////////////////////////////////////////////////////////
// global functions define
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool gc_ptr::operator==
		(
		const gc_element_ptr	&obj
		)const
{
	return ( normal_get() == obj.normal_get() ) ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool gc_ptr::operator!=
		(
		const gc_element_ptr	&obj
		)const
{
	return ( normal_get() != obj.normal_get() ) ? true : false;
}

//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool gc_element_ptr::operator==
		(
		const gc_ptr	&obj
		)const
{
	return ( normal_get() == obj.normal_get() ) ? true : false;
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool gc_element_ptr::operator!=
		(
		const gc_ptr	&obj
		)const
{
	return ( normal_get() != obj.normal_get() ) ? true : false;
}
///////////////////////////////////////////////////////////////////////////
// global functions define

//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		( 
		bool			exist , 
		const gc_ptr	&obj
		)
{
	return ( obj == exist );
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		( 
		bool			exist , 
		const gc_ptr	&obj
		)
{
	return ( obj != exist );
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator==
		( 
		bool					exist , 
		const gc_element_ptr	&obj
		)
{
	return ( obj == exist );
}
//=======================================================================
//!	compare
//-----------------------------------------------------------------------
inline
bool operator!=
		( 
		bool					exist , 
		const gc_element_ptr	&obj
		)
{
	return ( obj != exist );
}


};	//namespace

//using namespace NAMESPACE;	

#pragma pack( pop )			//release align

#endif

