/****************************************************************************/
/*!
   	@file		Real.h
	@author 	Fanzo
 	@date 		2007/10/18
*/
/****************************************************************************/

#ifndef		real_Real_h
#define		real_Real_h

///////////////////////////////////////////////////////////////////////////
//include files
#ifdef WIN32
	#include	<windows.h>
#elif __linux__
	#include	<pthread.h>
#endif

#pragma pack( push , 1 )		//set align

namespace real
{

///////////////////////////////////////////////////////////////////////////
// preprocessor deifne

#ifdef __APPLE__ 
	#define		real_call			pascal
	#define		real_extern			extern "C"
	#define		real_dll_export		__attribute__ ((visibility("default")))		/*extern "C" __declspec(dllexport)*/
	#define		real_dll_import		extern
//	#define		real_dll_export		extern "C"
	#define		real_tls			__thread
#elif __linux__
	#define		real_call			
	#define		real_extern			extern "C"
	#define		real_dll_export		__attribute__ ((visibility("default")))		/*extern "C" __declspec(dllexport)*/
	#define		real_dll_import		extern
	#define		real_tls			__thread
#elif WIN32
	#define		real_call			__cdecl
	#define		real_extern			extern "C"
	#define		real_dll_export		extern "C" __declspec(dllexport)
	#define		real_dll_import		extern "C" __declspec(dllimport)
	#define		real_tls			__declspec(thread)
#else
	#error		Unknown OS.
#endif

typedef		signed char				int8;
typedef		unsigned char			uint8;
typedef		signed short			int16;
typedef		unsigned short			uint16;
typedef		signed long				int32;
typedef		unsigned long			uint32;
typedef		signed long long		int64;
typedef		unsigned long long		uint64;
typedef		unsigned long			utf32;
typedef		unsigned short			utf16;
typedef		unsigned char			utf8;
typedef		char					sjis;
typedef		wchar_t					wchar;

#if defined(__APPLE__) || defined(__linux__)
	#define		real_align( a )		__attribute__((aligned(16)))
#elif WIN32
	#define		real_align( a )		__declspec( align( a ) )
#else
	#error		Unknown OS
#endif

///////////////////////////////////////////////////////////////////////////
// type define
enum GCThreadPriority
{
	Other_GCThreadPriority	= 0 , 
	
	Lowest_GCThreadPriority , 
	BelowNormal_GCThreadPriority , 
	Normal_GCThreadPriority , 
	AboveNormal_GCThreadPriority , 
	Highest_GCThreadPriority , 
};

///////////////////////////////////////////////////////////////////////////
// classes define
class CollectSetting
{
public:
	uint32			m_PollingTime;				//msec. if 0xFFFFFFFF , infinity.
	uint32			m_AsyncCollectMemorySize;	//if used memory is larger then this , execute collect async. if zero , not execute collect
	uint32			m_SyncCollectMemorySize;	//if used memory is larger then this , execute collect sync. if zero , not execute collect
	
	CollectSetting() : 
		m_PollingTime( 300 ) , 
		m_AsyncCollectMemorySize( 0 ) , 
		m_SyncCollectMemorySize( 0 )
	{
	}
};
///////////////////////////////////////////////////////////////////////////
// global variable define

///////////////////////////////////////////////////////////////////////////
// global functions define
struct gc_info;
//=======================================================================
//!	initialize
//!	@retval			---
//-----------------------------------------------------------------------
void real_call InitializeReal();
//=======================================================================
//!	uninitialize
//!	@retval			---
//-----------------------------------------------------------------------
void real_call UninitializeReal();
//=======================================================================
//!	regist 
//!	@retval			---
//-----------------------------------------------------------------------
bool real_call RegistRealGC
		(
		real::gc_info		*pinfo
		);
//=======================================================================
//!	execute collrect sync
//!	@retval			---
//-----------------------------------------------------------------------
void real_call CollectGC
		(
		bool	sync , 
		bool	integrate = false
		);
//=======================================================================
//!	get memory size
//!	@retval			---
//-----------------------------------------------------------------------
uint32 real_call GetMemorySize();
//=======================================================================
//!	set collect setting
//!	@retval			---
//-----------------------------------------------------------------------
void real_call SetCollectSetting
		(
		CollectSetting	&setting
		);
//=======================================================================
//!	get collect setting
//!	@retval			---
//-----------------------------------------------------------------------
void real_call GetCollectSetting
		(
		CollectSetting	*psetting
		);
//=======================================================================
//!	set gc thread priority
//!	@retval			---
//-----------------------------------------------------------------------
bool real_call SetGCThreadPriority
		(
		GCThreadPriority	priority
		);
//=======================================================================
//!	get gc thread priority
//!	@retval			---
//-----------------------------------------------------------------------
GCThreadPriority real_call GetGCThreadPriority();
//=======================================================================
//!	set gc thread processor
//!	@retval			---
//-----------------------------------------------------------------------
bool real_call SetGCThreadProcessor
		(
		int		processor_id
		);
};

///////////////////////////////////////////////////////////////////////////
// export

// export variable
#ifdef WIN32
	#ifndef REALDLL_EXPORTS
		real_dll_import
		DWORD __real_TlsIndex;
	#else
		real_dll_export
		DWORD __real_TlsIndex;
	#endif
#elif __APPLE__
//	#ifndef REALDLL_EXPORTS
		real_dll_import
		TaskStorageIndex __real_TlsIndex;
//	#endif
#elif __linux__
	real_dll_import
	pthread_key_t	__real_TlsIndex;
#endif


//using namespace NAMESPACE;	
#include	"Unit/Guid.h"
#include	"Debug.h"
#include	"Misc.h"
#include	"Atomic.h"
#include	"GC.h"
#include	"Object.h"

#pragma pack( pop )			//release align

#endif

