/****************************************************************************/
/*!
   	@file		Atomic.h
	@author 	Fanzo
 	@date 		2007/10/23
*/
/****************************************************************************/

#ifndef		real_Atomic_h
#define		real_Atomic_h

///////////////////////////////////////////////////////////////////////////
//include files

#pragma pack( push , 1 )		//set align

namespace real
{

///////////////////////////////////////////////////////////////////////////
// preprocessor deifne

///////////////////////////////////////////////////////////////////////////
// type define

///////////////////////////////////////////////////////////////////////////
// classes define

/****************************************************************************
"atomic_uint32" class 
****************************************************************************/
class atomic_uint32
{
	real_copy_impossible( atomic_uint32 );
public:
	uint8		m_Buffer[ sizeof( uint32 ) + sizeof( void* ) ];
	uint32*		m_Ptr;

public:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
atomic_uint32()
{
#ifdef WIN32
#pragma warning(push)
#pragma warning(disable: 4311)
#pragma warning(disable: 4312)
#endif
	m_Ptr	= ( uint32* )
		( ( unsigned int )( ( &m_Buffer[ 0 ] + sizeof( void* ) ) ) 
		& 
		( ~( sizeof( void* ) - 1 ) ) );
#ifdef WIN32
#pragma warning(pop)
#endif
	*m_Ptr	= 0;
}
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
atomic_uint32
		(
		uint32		val
		)
{
#ifdef WIN32
#pragma warning(push)
#pragma warning(disable: 4311)
#pragma warning(disable: 4312)
#endif
	m_Ptr	= ( uint32* )
		( ( unsigned int )( ( &m_Buffer[ 0 ] + sizeof( void* ) ) ) 
		& 
		( ~( sizeof( void* ) - 1 ) ) );
#ifdef WIN32
#pragma warning(pop)
#endif
	*m_Ptr	= val;
}
//=======================================================================
//!	initialize
//-----------------------------------------------------------------------
inline
void initialize
		(
		uint32		val
		)
{
#ifdef WIN32
#pragma warning(push)
#pragma warning(disable: 4311)
#pragma warning(disable: 4312)
#endif
	m_Ptr	= ( uint32* )
		( ( unsigned int )( ( &m_Buffer[ 0 ] + sizeof( void* ) ) ) 
		& 
		( ~( sizeof( void* ) - 1 ) ) );
#ifdef WIN32
#pragma warning(pop)
#endif
	*m_Ptr	= val;
}
//=======================================================================
//!	atomic set
//-----------------------------------------------------------------------
inline
void atomic_set
		(
		uint32	val
		)
{
#ifdef __GNUC__
	asm volatile
		(
		"lock; xchg %1 , (%0)		\t\n"
		: : "r"(m_Ptr) , "r"(val) : 
		);
#else	
	_asm
	{
		mov	ecx , this
		mov	ecx , [ ecx + m_Ptr ]
		mov	eax , val
		lock xchg [ ecx ] , eax
	}
#endif
}
//=======================================================================
//!	normal get
//-----------------------------------------------------------------------
inline
uint32 normal_get()
{
	return *m_Ptr;
}
//=======================================================================
//!	atomic set
//-----------------------------------------------------------------------
inline
void normal_set
		(
		uint32	val
		)
{
	*m_Ptr	= val;
}
//=======================================================================
//!	atomic get
//-----------------------------------------------------------------------
inline
uint32 atomic_get()
{
#ifdef __GNUC__
	uint32	ret;
	asm volatile
		(
		"lock; cmpxchg	%%eax , (%1)		\t\n"
		"mov				%%eax , %0		\t\n"
		: "=r"(ret) : "r"(m_Ptr) : "%eax" 
		);
	return ret;
#else	
	uint32	r;
	_asm
	{
		mov	ecx , this
		mov	ecx , [ ecx + m_Ptr ]
		lock cmpxchg [ ecx ] , eax
		mov	r , eax
	}
	return r;
#endif
}
//=======================================================================
//!	atomic increment
//-----------------------------------------------------------------------
inline
void atomic_increment()
{
#ifdef __GNUC__
	asm volatile
	(
		"lock; incl (%0)			\t\n"
		:  : "r"(m_Ptr) :  
	);
#else	
	__asm
	{
		mov	ecx , this
		mov	ecx , [ ecx + m_Ptr ]
		lock inc dword ptr [ ecx ]
	}
#endif
}
//=======================================================================
//!	atomic increment
//-----------------------------------------------------------------------
inline
uint32 atomic_increment_r()
{
#ifdef __GNUC__
	uint32	ret;
	asm volatile
		(
		"mov		$1 , %%eax				\t\n"	
		"lock; xadd	%%eax , (%1)			\t\n"
		"mov			%%eax , %0			\t\n"
		: "=r"(ret) : "r"(m_Ptr) : "%eax" 
		);
	return ret + 1;
#else	
	uint32		r;
	_asm
	{
		mov	ecx , this
		mov	ecx , [ ecx + m_Ptr ]
		mov	eax , 1
		lock xadd [ ecx ] , eax
		mov	r , eax
	}
	return r + 1;
#endif
}
//=======================================================================
//!	atomic decrement
//-----------------------------------------------------------------------
inline
void atomic_decrement()
{
#ifdef __GNUC__
	asm volatile
	(
		"lock; decl (%0)			\t\n"
		:  : "r"(m_Ptr) :  
	);
#else	
	_asm
	{
		mov	ecx , this
		mov	ecx , [ ecx + m_Ptr ]
		lock dec dword ptr [ ecx ]
	}
#endif
}
//=======================================================================
//!	atomic decrement
//-----------------------------------------------------------------------
inline
uint32 atomic_decrement_r()
{
#ifdef __GNUC__
	uint32	ret;
	asm volatile
		(
		"mov		$-1 , %%eax				\t\n"	
		"lock; xadd	%%eax , (%1)			\t\n"
		"mov			%%eax , %0			\t\n"
		: "=r"(ret) : "r"(m_Ptr) : "%eax" 
		);
	return ret - 1;
#else	
	uint32		r;
	_asm
	{
		mov	ecx , this
		mov	ecx , [ ecx + m_Ptr ]
		mov	eax , -1
		lock xadd [ ecx ] , eax
		mov	r , eax
	}
	return r - 1;
#endif
}
//=======================================================================
//!	atomic add
//-----------------------------------------------------------------------
inline
void atomic_add
		(
		int32	val
		)
{
#ifdef __GNUC__
	asm volatile
		(
		"lock; add %1 , (%0)		\t\n"	
		: : "r"(m_Ptr) , "r"(val) : 
		);
#else	
	_asm
	{
		mov	ecx , this
		mov	ecx , [ ecx + m_Ptr ]
		mov	eax , val
		lock add [ ecx ] , eax
	}
#endif
}
//=======================================================================
//!	atomic decrement
//-----------------------------------------------------------------------
inline
void atomic_sub
		(
		int32	val
		)
{
#ifdef __GNUC__
	asm volatile
		(
		"lock; sub %1 , (%0)		\t\n"	
		: : "r"(m_Ptr) , "r"(val) : 
		);
#else	
	_asm
	{
		mov	ecx , this
		mov	ecx , [ ecx ].m_Ptr
		mov	eax , val
		lock sub [ ecx ] , eax
	}
#endif
}
//=======================================================================
//!	atomic or
//-----------------------------------------------------------------------
inline
void atomic_or
		(
		uint32		val
		)
{
#ifdef __GNUC__
	asm volatile
		(
		"lock; or %1 , (%0)	\t\n"	
		: : "r"(m_Ptr) , "r"(val) : 
		);
#else	
	_asm
	{
		mov	ecx , this
		mov	ecx , [ ecx + m_Ptr ]
		mov	eax , val
		lock or [ ecx ] , eax
	}
#endif
}
//=======================================================================
//!	atomic and
//-----------------------------------------------------------------------
inline
void atomic_and
		(
		uint32		val
		)
{
#ifdef __GNUC__
	asm volatile
		(
		"lock; and %1 , (%0)	\t\n"	
		: : "r"(m_Ptr) , "r"(val) : 
		);
#else	
	_asm
	{
		mov	ecx , this
		mov	ecx , [ ecx + m_Ptr ]
		mov	eax , val
		lock and [ ecx ] , eax
	}
#endif
}
};

/****************************************************************************
"atomic_ptr" class 
****************************************************************************/
template< class PTR_CLASS >
class atomic_ptr
{
	real_copy_impossible( atomic_ptr );
	
private:
	uint8		m_Buffer[ sizeof( void* ) * 2 ];
	PTR_CLASS**	m_Ptr;
	
public:
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
atomic_ptr()
{
#ifdef WIN32
#pragma warning(push)
#pragma warning(disable: 4311)
#pragma warning(disable: 4312)
#endif
	m_Ptr	= ( PTR_CLASS** )
		( ( unsigned int )( ( &m_Buffer[ 0 ] + sizeof( void* ) ) ) 
		& 
		( ~( sizeof( void* ) - 1 ) ) );
#ifdef WIN32
#pragma warning(pop)
#endif
	*m_Ptr	= 0;
}
//=======================================================================
//!	construct
//-----------------------------------------------------------------------
atomic_ptr
		(
		PTR_CLASS	*ptr
		)
{
#ifdef WIN32
#pragma warning(push)
#pragma warning(disable: 4311)
#pragma warning(disable: 4312)
#endif
	m_Ptr	= ( PTR_CLASS** )
		( ( unsigned int )( ( &m_Buffer[ 0 ] + sizeof( void* ) ) ) 
		& 
		( ~( sizeof( void* ) - 1 ) ) );
#ifdef WIN32
#pragma warning(pop)
#endif
	*m_Ptr	= ptr;
}
//=======================================================================
//!	initialize
//-----------------------------------------------------------------------
inline
void initialize
		(
		PTR_CLASS	*ptr
		)
{
#ifdef WIN32
#pragma warning(push)
#pragma warning(disable: 4311)
#pragma warning(disable: 4312)
#endif
	m_Ptr	= ( PTR_CLASS** )
		( ( unsigned int )( ( &m_Buffer[ 0 ] + sizeof( void* ) ) ) 
		& 
		( ~( sizeof( void* ) - 1 ) ) );
#ifdef WIN32
#pragma warning(pop)
#endif
	*m_Ptr	= ptr;
}
//=======================================================================
//!	atomic_set
//-----------------------------------------------------------------------
inline
void atomic_set
		(
		PTR_CLASS	*t_ptr
		)
{
#ifdef __GNUC__
	asm volatile
		(
		"lock; xchg %1 , (%0)		\t\n"
		: : "r"(m_Ptr) , "r"(t_ptr) : 
		);
#else	
	_asm
	{
		mov	ecx , this
		mov	ecx , [ ecx + m_Ptr ]
		mov	eax , t_ptr
		lock xchg [ ecx ] , eax
	}
#endif
}
//=======================================================================
//!	atomic_get
//-----------------------------------------------------------------------
inline
PTR_CLASS* atomic_get()const
{
#ifdef __GNUC__
	PTR_CLASS*	ret;
	asm volatile
		(
		"lock; cmpxchg	%%eax , (%1)		\t\n"
		"mov				%%eax , %0		\t\n"
		: "=r"(ret) : "r"(m_Ptr) : "%eax" 
		);
	return ret;
#else	
	PTR_CLASS*	r;
	_asm
	{
		mov	ecx , this
		mov	ecx , [ ecx + m_Ptr ]
		lock cmpxchg [ ecx ] , eax
		mov	r , eax
	}
	return r;
#endif
}
//=======================================================================
//!	normal_set
//-----------------------------------------------------------------------
inline
void normal_set
		(
		PTR_CLASS	*ptr
		)
{
	*m_Ptr	= ptr;
}
//=======================================================================
//!	normal_set
//-----------------------------------------------------------------------
inline
PTR_CLASS* normal_get()const
{
	return *m_Ptr;
}
};

///////////////////////////////////////////////////////////////////////////
// global variable define

///////////////////////////////////////////////////////////////////////////
// global functions define

};	//namespace

//using namespace real;		

#pragma pack( pop )			//release align

#endif

